@@ -390,8 +390,24 @@ void CG_DrawFlagModel( float x, float y, float w, float h, int team, qboolean fo
 
 		if( team == TEAM_RED ) {
 			handle = cgs.media.redFlagModel;
+			if(cgs.gametype == GT_DOUBLE_D){
+				if(cgs.redflag == TEAM_BLUE)
+					handle = cgs.media.blueFlagModel;
+				if(cgs.redflag == TEAM_FREE)
+					handle = cgs.media.neutralFlagModel;
+				if(cgs.redflag == TEAM_NONE)
+					handle = cgs.media.neutralFlagModel;
+			}
 		} else if( team == TEAM_BLUE ) {
 			handle = cgs.media.blueFlagModel;
+			if(cgs.gametype == GT_DOUBLE_D){
+				if(cgs.redflag == TEAM_BLUE)
+					handle = cgs.media.blueFlagModel;
+				if(cgs.redflag == TEAM_FREE)
+					handle = cgs.media.neutralFlagModel;
+				if(cgs.redflag == TEAM_NONE)
+					handle = cgs.media.neutralFlagModel;
+			}
 		} else if( team == TEAM_FREE ) {
 			handle = cgs.media.neutralFlagModel;
 		} else {
@@ -530,9 +546,7 @@ static void CG_DrawStatusBar( void ) {
 	vec4_t		hcolor;
 	vec3_t		angles;
 	vec3_t		origin;
-#ifdef MISSIONPACK
-	qhandle_t	handle;
-#endif
+        
 	static float colors[4][4] = { 
 //		{ 0.2, 1.0, 0.2, 1.0 } , { 1.0, 0.2, 0.2, 1.0 }, {0.5, 0.5, 0.5, 1} };
 		{ 1.0f, 0.69f, 0.0f, 1.0f },    // normal
@@ -545,7 +559,10 @@ static void CG_DrawStatusBar( void ) {
 	}
 
 	// draw the team background
-	CG_DrawTeamBackground( 0, 420, 640, 60, 0.33f, cg.snap->ps.persistant[PERS_TEAM] );
+	if ( !(cg.snap->ps.pm_flags & PMF_FOLLOW) ) //If not following anybody:
+		CG_DrawTeamBackground( 0, 420, 640, 60, 0.33f, cg.snap->ps.persistant[PERS_TEAM] );
+	else //Sago: If we follow find the teamcolor of the guy we follow. It might not be our own team!
+		CG_DrawTeamBackground( 0, 420, 640, 60, 0.33f, cgs.clientinfo[ cg.snap->ps.clientNum ].team );
 
 	cent = &cg_entities[cg.snap->ps.clientNum];
 	ps = &cg.snap->ps;
@@ -580,20 +597,6 @@ static void CG_DrawStatusBar( void ) {
 		CG_Draw3DModel( 370 + CHAR_WIDTH*3 + TEXT_ICON_SPACE, 432, ICON_SIZE, ICON_SIZE,
 					   cgs.media.armorModel, 0, origin, angles );
 	}
-#ifdef MISSIONPACK
-	if( cgs.gametype == GT_HARVESTER ) {
-		origin[0] = 90;
-		origin[1] = 0;
-		origin[2] = -10;
-		angles[YAW] = ( cg.time & 2047 ) * 360 / 2048.0;
-		if( cg.snap->ps.persistant[PERS_TEAM] == TEAM_BLUE ) {
-			handle = cgs.media.redCubeModel;
-		} else {
-			handle = cgs.media.blueCubeModel;
-		}
-		CG_Draw3DModel( 640 - (TEXT_ICON_SPACE + ICON_SIZE), 416, ICON_SIZE, ICON_SIZE, handle, 0, origin, angles );
-	}
-#endif
 	//
 	// ammo
 	//
@@ -663,29 +666,6 @@ static void CG_DrawStatusBar( void ) {
 		}
 
 	}
-#ifdef MISSIONPACK
-	//
-	// cubes
-	//
-	if( cgs.gametype == GT_HARVESTER ) {
-		value = ps->generic1;
-		if( value > 99 ) {
-			value = 99;
-		}
-		trap_R_SetColor( colors[0] );
-		CG_DrawField (640 - (CHAR_WIDTH*2 + TEXT_ICON_SPACE + ICON_SIZE), 432, 2, value);
-		trap_R_SetColor( NULL );
-		// if we didn't draw a 3D icon, draw a 2D icon for armor
-		if ( !cg_draw3dIcons.integer && cg_drawIcons.integer ) {
-			if( cg.snap->ps.persistant[PERS_TEAM] == TEAM_BLUE ) {
-				handle = cgs.media.redCubeIcon;
-			} else {
-				handle = cgs.media.blueCubeIcon;
-			}
-			CG_DrawPic( 640 - (TEXT_ICON_SPACE + ICON_SIZE), 432, ICON_SIZE, ICON_SIZE, handle );
-		}
-	}
-#endif
 }
 #endif
 
@@ -834,6 +814,289 @@ static float CG_DrawTimer( float y ) {
 	return y + BIGCHAR_HEIGHT + 4;
 }
 
+/*
+CG_DrawDoubleDominationThings
+*/
+
+static float CG_DrawDoubleDominationThings( float y ) {
+	char		*s;
+	int			w;
+	int 		statusA, statusB;
+	statusA = cgs.redflag;
+	statusB = cgs.blueflag;
+
+	if(statusA == TEAM_NONE) {
+		s = va("Point A not spawned");
+	} else
+	if(statusA == TEAM_FREE) {
+		s = va("Point A is not controlled");
+	} else
+	if(statusA == TEAM_RED) {
+		s = va("Point A is controlled by RED");
+	} else
+	if(statusA == TEAM_BLUE) {
+		s = va("Point A is controlled by BLUE");
+	} else
+		s = va("Point A has an error");
+	w = CG_DrawStrlen( s ) * SMALLCHAR_WIDTH;
+	CG_DrawSmallString( 635 - w, y + 2, s, 1.0F);
+	y+=SMALLCHAR_HEIGHT+4;
+
+	if(statusB == TEAM_NONE) {
+		s = va("Point B not spawned");
+	} else
+	if(statusB == TEAM_FREE) {
+		s = va("Point B is not controlled");
+	} else
+	if(statusB == TEAM_RED) {
+		s = va("Point B is controlled by RED");
+	} else
+	if(statusB == TEAM_BLUE) {
+		s = va("Point B is controlled by BLUE");
+	} else
+		s = va("Point B has an error");
+	w = CG_DrawStrlen( s ) * SMALLCHAR_WIDTH;
+	CG_DrawSmallString( 635 - w, y + 2, s, 1.0F);
+
+	if( ( ( statusB == statusA ) && ( statusA == TEAM_RED ) ) ||
+		( ( statusB == statusA ) && ( statusA == TEAM_BLUE ) ) ) {
+		s = va("Capture in: %i",(cgs.timetaken+10*1000-cg.time)/1000+1);
+		w = CG_DrawStrlen( s ) * SMALLCHAR_WIDTH;
+		y+=SMALLCHAR_HEIGHT+4;
+		CG_DrawSmallString( 635 - w, y + 2, s, 1.0F);
+	}
+
+	return y + SMALLCHAR_HEIGHT+4;
+}
+
+/*
+=================
+CG_DrawLMSmode
+=================
+*/
+
+static float CG_DrawLMSmode( float y ) {
+	char		*s;
+	int		w;
+
+	if(cgs.lms_mode == 0) {
+		s = va("LMS: Point/round + OT");
+	} else
+	if(cgs.lms_mode == 1) {
+		s = va("LMS: Point/round - OT");
+	} else
+	if(cgs.lms_mode == 2) {
+		s = va("LMS: Point/kill + OT");
+	} else
+	if(cgs.lms_mode == 3) {
+		s = va("LMS: Point/kill - OT");
+	} else
+		s = va("LMS: Unknown mode");
+
+	w = CG_DrawStrlen( s ) * SMALLCHAR_WIDTH;
+	CG_DrawSmallString( 635 - w, y + 2, s, 1.0F);
+
+	return y + SMALLCHAR_HEIGHT+4;
+}
+
+/*
+=================
+CG_DrawCTFoneway
+=================
+*/
+
+static float CG_DrawCTFoneway( float y ) {
+	char		*s;
+	int		w;
+	vec4_t		color;
+
+	if(cgs.gametype != GT_CTF_ELIMINATION)
+		return y;
+
+	memcpy(color,g_color_table[ColorIndex(COLOR_WHITE)],sizeof(color));
+
+	if(cgs.oneway == 0) {
+		return y; //nothing to draw
+	} else
+	if(cgs.attackingTeam == TEAM_BLUE) {
+		memcpy(color,g_color_table[ColorIndex(COLOR_BLUE)],sizeof(color));
+		s = va("Blue team on offence");
+	} else
+	if(cgs.attackingTeam == TEAM_RED) {
+		memcpy(color,g_color_table[ColorIndex(COLOR_RED)],sizeof(color));
+		s = va("Red team on offence");
+	} else
+		s = va("Unknown team on offence");
+
+	w = CG_DrawStrlen( s ) * SMALLCHAR_WIDTH;
+	CG_DrawSmallStringColor( 635 - w, y + 2, s, color);
+
+	return y + SMALLCHAR_HEIGHT+4;
+}
+
+/*
+=================
+CG_DrawEliminationDeathMessage
+=================
+*/
+
+/*static float CG_DrawEliminationDeathMessage( float y ) {
+	char		*s;
+	int			w;
+
+	s = va("You are waiting for a new round");
+	w = CG_DrawStrlen( s ) * SMALLCHAR_WIDTH;
+	CG_DrawSmallString( 635 - w, y + 2, s, 1.0F);
+
+	return y + SMALLCHAR_HEIGHT+4;
+}*/
+
+/*
+=================
+CG_DrawDomStatus
+=================
+*/
+
+static float CG_DrawDomStatus( float y ) {
+	int 		i,w; 
+	char		*s;
+	vec4_t		color;
+	
+	for(i = 0;i < cgs.domination_points_count;i++) {
+		switch(cgs.domination_points_status[i]) {
+			case TEAM_RED:
+				memcpy(color,g_color_table[ColorIndex(COLOR_RED)],sizeof(color));
+				break;
+			case TEAM_BLUE:
+				memcpy(color,g_color_table[ColorIndex(COLOR_BLUE)],sizeof(color));
+				break;
+			default:
+				memcpy(color,g_color_table[ColorIndex(COLOR_WHITE)],sizeof(color));
+				break;			
+		}
+		
+		s = va("%s",cgs.domination_points_names[i]);
+		w = CG_DrawStrlen( s ) * SMALLCHAR_WIDTH;
+		CG_DrawSmallStringColor( 635 - w, y + 2, s, color);
+		y += SMALLCHAR_HEIGHT+4;
+			
+	}
+
+	return y;
+}
+
+
+/*
+=================
+CG_DrawEliminationTimer
+=================
+*/
+static float CG_DrawEliminationTimer( float y ) {
+	char		*s;
+	int			w;
+	int			mins, seconds, tens, sec;
+	int			msec;
+	vec4_t			color;
+	const char	*st;
+	float scale;
+	int cw;
+	int rst;
+	
+	rst = cgs.roundStartTime;
+
+	//default color is white
+	memcpy(color,g_color_table[ColorIndex(COLOR_WHITE)],sizeof(color));
+
+	//msec = cg.time - cgs.levelStartTime;
+	if(cg.time>rst) //We are started
+	{
+		msec = cgs.roundtime*1000 - (cg.time -rst);
+		if(msec<=30*1000-1) //<= 30 seconds
+			memcpy(color,g_color_table[ColorIndex(COLOR_YELLOW)],sizeof(color));
+		if(msec<=10*1000-1) //<= 10 seconds
+			memcpy(color,g_color_table[ColorIndex(COLOR_RED)],sizeof(color));
+		msec += 1000; //120-1 instead of 119-0
+	}
+	else
+	{
+		//Warmup
+		msec = -cg.time +rst;
+		memcpy(color,g_color_table[ColorIndex(COLOR_GREEN)],sizeof(color));
+		sec = msec/1000;
+		msec += 1000; //5-1 instead of 4-0
+/***
+Lots of stuff
+****/
+	if(cg.warmup == 0)
+	{
+		st = va( "Round in: %i", sec + 1 );
+		if ( sec != cg.warmupCount ) {
+			cg.warmupCount = sec;
+			switch ( sec ) {
+			case 0:
+				trap_S_StartLocalSound( cgs.media.count1Sound, CHAN_ANNOUNCER );
+				break;
+			case 1:
+				trap_S_StartLocalSound( cgs.media.count2Sound, CHAN_ANNOUNCER );
+				break;
+			case 2:
+				trap_S_StartLocalSound( cgs.media.count3Sound, CHAN_ANNOUNCER );
+				break;
+			default:
+				break;
+			}
+		} 
+		scale = 0.45f;
+		switch ( cg.warmupCount ) {
+		case 0:
+			cw = 28;
+			scale = 0.54f;
+			break;
+		case 1:
+			cw = 24;
+			scale = 0.51f;
+			break;
+		case 2:
+			cw = 20;
+			scale = 0.48f;
+			break;
+		default:
+			cw = 16;
+			scale = 0.45f;
+			break;
+		}
+
+	#ifdef MISSIONPACK
+			//w = CG_Text_Width(s, scale, 0);
+			//CG_Text_Paint(320 - w / 2, 125, scale, colorWhite, st, 0, 0, ITEM_TEXTSTYLE_SHADOWEDMORE);
+	#else
+		w = CG_DrawStrlen( st );
+		CG_DrawStringExt( 320 - w * cw/2, 70, st, colorWhite, 
+				qfalse, qtrue, cw, (int)(cw * 1.5), 0 );
+	#endif
+	}
+/*
+Lots of stuff
+*/
+	}
+
+	seconds = msec / 1000;
+	mins = seconds / 60;
+	seconds -= mins * 60;
+	tens = seconds / 10;
+	seconds -= tens * 10;
+
+	if(msec>=0)
+		s = va( " %i:%i%i", mins, tens, seconds );
+	else
+		s = va( " Overtime");
+	w = CG_DrawStrlen( s ) * BIGCHAR_WIDTH;
+	
+	CG_DrawBigStringColor( 635 - w, y + 2, s, color);
+
+	return y + BIGCHAR_HEIGHT + 4;
+}
+
 
 /*
 =================
@@ -1008,6 +1271,21 @@ static float CG_DrawTeamOverlay( float y, qboolean right, qboolean upper ) {
 //#endif
 }
 
+static float CG_DrawFollowMessage( float y ) {
+	char		*s;
+	int			w;	
+
+	if ( !(cg.snap->ps.pm_flags & PMF_FOLLOW) ) {
+		return y;
+	}
+
+	s = va("USE_ITEM to stop following");
+	w = CG_DrawStrlen( s ) * SMALLCHAR_WIDTH;
+	CG_DrawSmallString( 635 - w, y + 2, s, 1.0F);
+
+	return y + SMALLCHAR_HEIGHT+4;
+}
+
 
 /*
 =====================
@@ -1020,16 +1298,40 @@ static void CG_DrawUpperRight( void ) {
 
 	y = 0;
 
-	if ( cgs.gametype >= GT_TEAM && cg_drawTeamOverlay.integer == 1 ) {
+	if ( cgs.gametype >= GT_TEAM && cgs.ffa_gt!=1 && cg_drawTeamOverlay.integer == 1 ) {
 		y = CG_DrawTeamOverlay( y, qtrue, qtrue );
+	}
+	if ( cgs.gametype == GT_DOUBLE_D ) {
+		y = CG_DrawDoubleDominationThings(y);
 	} 
+	else
+	if ( cgs.gametype == GT_LMS && cg.showScores ) {
+		y = CG_DrawLMSmode(y);
+	}
+	else
+	if ( cgs.gametype == GT_CTF_ELIMINATION ) {
+		y = CG_DrawCTFoneway(y);
+	}
+	else
+	if ( cgs.gametype == GT_DOMINATION ) {
+		y = CG_DrawDomStatus(y);
+	}
+	
 	if ( cg_drawSnapshot.integer ) {
 		y = CG_DrawSnapshot( y );
 	}
 	if ( cg_drawFPS.integer ) {
 		y = CG_DrawFPS( y );
 	}
-	if ( cg_drawTimer.integer ) {
+	if (cgs.gametype==GT_ELIMINATION || cgs.gametype == GT_CTF_ELIMINATION || cgs.gametype==GT_LMS) {
+		y = CG_DrawEliminationTimer( y );
+		/*if (cgs.clientinfo[ cg.clientNum ].isDead)
+			y = CG_DrawEliminationDeathMessage( y);*/
+	}
+
+	y = CG_DrawFollowMessage( y );
+
+	if ( cg_drawTimer.integer) {
 		y = CG_DrawTimer( y );
 	}
 	if ( cg_drawAttacker.integer ) {
@@ -1071,7 +1373,7 @@ static float CG_DrawScores( float y ) {
 	y1 = y;
 
 	// draw from the right side to left
-	if ( cgs.gametype >= GT_TEAM ) {
+	if ( cgs.gametype >= GT_TEAM && cgs.ffa_gt!=1) {
 		x = 640;
 		color[0] = 0.0f;
 		color[1] = 0.0f;
@@ -1086,7 +1388,7 @@ static float CG_DrawScores( float y ) {
 		}
 		CG_DrawBigString( x + 4, y, s, 1.0F);
 
-		if ( cgs.gametype == GT_CTF ) {
+		if ( cgs.gametype == GT_CTF || cgs.gametype == GT_CTF_ELIMINATION) {
 			// Display flag status
 			item = BG_FindItemForPowerup( PW_BLUEFLAG );
 
@@ -1110,7 +1412,7 @@ static float CG_DrawScores( float y ) {
 		}
 		CG_DrawBigString( x + 4, y, s, 1.0F);
 
-		if ( cgs.gametype == GT_CTF ) {
+		if ( cgs.gametype == GT_CTF || cgs.gametype == GT_CTF_ELIMINATION ) {
 			// Display flag status
 			item = BG_FindItemForPowerup( PW_REDFLAG );
 
@@ -1122,20 +1424,7 @@ static float CG_DrawScores( float y ) {
 			}
 		}
 
-#ifdef MISSIONPACK
-		if ( cgs.gametype == GT_1FCTF ) {
-			// Display flag status
-			item = BG_FindItemForPowerup( PW_NEUTRALFLAG );
-
-			if (item) {
-				y1 = y - BIGCHAR_HEIGHT - 8;
-				if( cgs.flagStatus >= 0 && cgs.flagStatus <= 3 ) {
-					CG_DrawPic( x, y1-4, w, BIGCHAR_HEIGHT+8, cgs.media.flagShader[cgs.flagStatus] );
-				}
-			}
-		}
-#endif
-		if ( cgs.gametype >= GT_CTF ) {
+		if ( cgs.gametype >= GT_CTF && cgs.ffa_gt==0) {
 			v = cgs.capturelimit;
 		} else {
 			v = cgs.fraglimit;
@@ -1327,7 +1616,7 @@ static void CG_DrawLowerRight( void ) {
 
 	y = 480 - ICON_SIZE;
 
-	if ( cgs.gametype >= GT_TEAM && cg_drawTeamOverlay.integer == 2 ) {
+	if ( cgs.gametype >= GT_TEAM && cgs.ffa_gt!=1 && cg_drawTeamOverlay.integer == 2 ) {
 		y = CG_DrawTeamOverlay( y, qtrue, qfalse );
 	} 
 
@@ -1380,7 +1669,7 @@ static void CG_DrawLowerLeft( void ) {
 
 	y = 480 - ICON_SIZE;
 
-	if ( cgs.gametype >= GT_TEAM && cg_drawTeamOverlay.integer == 3 ) {
+	if ( cgs.gametype >= GT_TEAM && cgs.ffa_gt!=1 && cg_drawTeamOverlay.integer == 3 ) {
 		y = CG_DrawTeamOverlay( y, qfalse, qfalse );
 	} 
 
@@ -2060,7 +2349,7 @@ static void CG_DrawSpectator(void) {
 	if ( cgs.gametype == GT_TOURNAMENT ) {
 		CG_DrawBigString(320 - 15 * 8, 460, "waiting to play", 1.0F);
 	}
-	else if ( cgs.gametype >= GT_TEAM ) {
+	else if ( cgs.gametype >= GT_TEAM && cgs.ffa_gt!=1) {
 		CG_DrawBigString(320 - 39 * 8, 460, "press ESC and use the JOIN menu to play", 1.0F);
 	}
 }
@@ -2178,7 +2467,7 @@ static qboolean CG_DrawScoreboard( void ) {
 
 
 	if (menuScoreboard == NULL) {
-		if ( cgs.gametype >= GT_TEAM ) {
+		if ( cgs.gametype >= GT_TEAM && cgs.ffa_gt!=1) {
 			menuScoreboard = Menus_FindByName("teamscore_menu");
 		} else {
 			menuScoreboard = Menus_FindByName("score_menu");
@@ -2267,6 +2556,10 @@ static void CG_DrawAmmoWarning( void ) {
 	const char	*s;
 	int			w;
 
+	//Don't report in instant gib same with RA
+	if(cgs.instantgib || cgs.rockets)
+		return;
+
 	if ( cg_drawAmmoWarning.integer == 0 ) {
 		return;
 	}
@@ -2390,6 +2683,14 @@ static void CG_DrawWarmup( void ) {
 			s = "Team Deathmatch";
 		} else if ( cgs.gametype == GT_CTF ) {
 			s = "Capture the Flag";
+		} else if ( cgs.gametype == GT_ELIMINATION ) {
+			s = "Elimination";
+		} else if ( cgs.gametype == GT_CTF_ELIMINATION ) {
+			s = "CTF Elimination";
+		} else if ( cgs.gametype == GT_LMS ) {
+			s = "Last Man Standing";
+		} else if ( cgs.gametype == GT_DOUBLE_D ) {
+			s = "Double Domination";
 #ifdef MISSIONPACK
 		} else if ( cgs.gametype == GT_1FCTF ) {
 			s = "One Flag CTF";
@@ -2516,7 +2817,7 @@ static void CG_Draw2D( void ) {
 		return;
 	}
 */
-	if ( cg.snap->ps.persistant[PERS_TEAM] == TEAM_SPECTATOR ) {
+	if ( cg.snap->ps.persistant[PERS_TEAM] == TEAM_SPECTATOR /*|| cg.snap->ps.pm_type == PM_SPECTATOR*/ ) {
 		CG_DrawSpectator();
 		CG_DrawCrosshair();
 		CG_DrawCrosshairNames();
@@ -2550,7 +2851,7 @@ static void CG_Draw2D( void ) {
 			CG_DrawReward();
 		}
     
-		if ( cgs.gametype >= GT_TEAM ) {
+		if ( cgs.gametype >= GT_TEAM && cgs.ffa_gt!=1) {
 #ifndef MISSIONPACK
 			CG_DrawTeamInfo();
 #endif
