@@ -118,35 +118,38 @@ static void CG_Obituary( entityState_t *ent ) {
 
 	// check for single client messages
 
-	switch( mod ) {
-	case MOD_SUICIDE:
-		message = "suicides";
-		break;
-	case MOD_FALLING:
-		message = "cratered";
-		break;
-	case MOD_CRUSH:
-		message = "was squished";
-		break;
-	case MOD_WATER:
-		message = "sank like a rock";
-		break;
-	case MOD_SLIME:
-		message = "melted";
-		break;
-	case MOD_LAVA:
-		message = "does a back flip into the lava";
-		break;
-	case MOD_TARGET_LASER:
-		message = "saw the light";
-		break;
-	case MOD_TRIGGER_HURT:
-		message = "was in the wrong place";
-		break;
-	default:
-		message = NULL;
-		break;
-	}
+        if(attacker != ENTITYNUM_WORLD)
+            message = NULL;
+        else
+            switch( mod ) {
+            case MOD_SUICIDE:
+                    message = "suicides";
+                    break;
+            case MOD_FALLING:
+                    message = "cratered";
+                    break;
+            case MOD_CRUSH:
+                    message = "was squished";
+                    break;
+            case MOD_WATER:
+                    message = "sank like a rock";
+                    break;
+            case MOD_SLIME:
+                    message = "melted";
+                    break;
+            case MOD_LAVA:
+                    message = "does a back flip into the lava";
+                    break;
+            case MOD_TARGET_LASER:
+                    message = "saw the light";
+                    break;
+            case MOD_TRIGGER_HURT:
+                    message = "was in the wrong place";
+                    break;
+            default:
+                    message = NULL;
+                    break;
+            }
 
 	if (attacker == target) {
 		gender = ci->gender;
@@ -209,6 +212,7 @@ static void CG_Obituary( entityState_t *ent ) {
 		CG_Printf( "%s %s.\n", targetName, message);
 		return;
 	}
+        
 
 	// check for kill messages from the current clientNum
 	if ( attacker == cg.snap->ps.clientNum ) {
@@ -318,6 +322,18 @@ static void CG_Obituary( entityState_t *ent ) {
 			message = "tried to invade";
 			message2 = "'s personal space";
 			break;
+                case MOD_LAVA:
+                        message = "was given a hot bath by";
+                        break;
+                case MOD_SLIME:
+                        message = "was given a acid bath by";
+                        break;
+                case MOD_FALLING:
+                        message = "was given a small push by";
+                        break;
+                case MOD_TRIGGER_HURT:
+                        message = "was helped on the way by";
+                        break;
 		default:
 			message = "was killed by";
 			break;
@@ -923,28 +939,77 @@ void CG_EntityEvent( centity_t *cent, vec3_t position ) {
 	case EV_RAILTRAIL:
 		DEBUGNAME("EV_RAILTRAIL");
 		cent->currentState.weapon = WP_RAILGUN;
-		// if the end was on a nomark surface, don't make an explosion
-		CG_RailTrail( ci, es->origin2, es->pos.trBase );
-		if ( es->eventParm != 255 ) {
-			ByteToDir( es->eventParm, dir );
-			CG_MissileHitWall( es->weapon, es->clientNum, position, dir, IMPACTSOUND_DEFAULT );
+//unlagged - attack prediction #2
+		// if the client is us, unlagged is on server-side, and we've got it client-side
+		if ( es->clientNum == cg.predictedPlayerState.clientNum && 
+				cgs.delagHitscan && (cg_delag.integer & 1 || cg_delag.integer & 16) ) {
+			// do nothing, because it was already predicted
+			//Com_Printf("Ignoring rail trail event\n");
 		}
+		else {
+			// draw a rail trail, because it wasn't predicted
+			CG_RailTrail( ci, es->origin2, es->pos.trBase );
+
+			// if the end was on a nomark surface, don't make an explosion
+			if ( es->eventParm != 255 ) {
+				ByteToDir( es->eventParm, dir );
+				CG_MissileHitWall( es->weapon, es->clientNum, position, dir, IMPACTSOUND_DEFAULT );
+			}
+			//Com_Printf("Non-predicted rail trail\n");
+		}
+//unlagged - attack prediction #2
 		break;
 
 	case EV_BULLET_HIT_WALL:
 		DEBUGNAME("EV_BULLET_HIT_WALL");
-		ByteToDir( es->eventParm, dir );
-		CG_Bullet( es->pos.trBase, es->otherEntityNum, dir, qfalse, ENTITYNUM_WORLD );
+//unlagged - attack prediction #2
+		// if the client is us, unlagged is on server-side, and we've got it client-side
+		if ( es->clientNum == cg.predictedPlayerState.clientNum && 
+				cgs.delagHitscan && (cg_delag.integer & 1 || cg_delag.integer & 2) ) {
+			// do nothing, because it was already predicted
+			//Com_Printf("Ignoring bullet event\n");
+		}
+		else {
+			// do the bullet, because it wasn't predicted
+			ByteToDir( es->eventParm, dir );
+			CG_Bullet( es->pos.trBase, es->otherEntityNum, dir, qfalse, ENTITYNUM_WORLD );
+			//Com_Printf("Non-predicted bullet\n");
+		}
+//unlagged - attack prediction #2
 		break;
 
 	case EV_BULLET_HIT_FLESH:
 		DEBUGNAME("EV_BULLET_HIT_FLESH");
-		CG_Bullet( es->pos.trBase, es->otherEntityNum, dir, qtrue, es->eventParm );
+//unlagged - attack prediction #2
+		// if the client is us, unlagged is on server-side, and we've got it client-side
+		if ( es->clientNum == cg.predictedPlayerState.clientNum && 
+				cgs.delagHitscan && (cg_delag.integer & 1 || cg_delag.integer & 2) ) {
+			// do nothing, because it was already predicted
+			//Com_Printf("Ignoring bullet event\n");
+		}
+		else {
+			// do the bullet, because it wasn't predicted
+			CG_Bullet( es->pos.trBase, es->otherEntityNum, dir, qtrue, es->eventParm );
+			//Com_Printf("Non-predicted bullet\n");
+		}
+//unlagged - attack prediction #2
 		break;
 
 	case EV_SHOTGUN:
 		DEBUGNAME("EV_SHOTGUN");
-		CG_ShotgunFire( es );
+//unlagged - attack prediction #2
+		// if the client is us, unlagged is on server-side, and we've got it client-side
+		if ( es->otherEntityNum == cg.predictedPlayerState.clientNum && 
+				cgs.delagHitscan && (cg_delag.integer & 1 || cg_delag.integer & 4) ) {
+			// do nothing, because it was already predicted
+			//Com_Printf("Ignoring shotgun event\n");
+		}
+		else {
+			// do the shotgun pattern, because it wasn't predicted
+			CG_ShotgunFire( es );
+			//Com_Printf("Non-predicted shotgun pattern\n");
+		}
+//unlagged - attack prediction #2
 		break;
 
 	case EV_GENERAL_SOUND:
