@@ -85,8 +85,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #define	DEFAULT_MODEL			"sarge"
 #ifdef MISSIONPACK
-#define	DEFAULT_TEAM_MODEL		"james"
-#define	DEFAULT_TEAM_HEAD		"*james"
+#define	DEFAULT_TEAM_MODEL		"sergei"
+#define	DEFAULT_TEAM_HEAD		"*sergei"
 #else
 #define	DEFAULT_TEAM_MODEL		"sarge"
 #define	DEFAULT_TEAM_HEAD		"sarge"
@@ -301,6 +301,7 @@ typedef struct {
 	int				captures;
 	qboolean	perfect;
 	int				team;
+	int			isDead;
 } score_t;
 
 // each client has an associated clientInfo_t
@@ -374,6 +375,8 @@ typedef struct {
 	animation_t		animations[MAX_TOTALANIMATIONS];
 
 	sfxHandle_t		sounds[MAX_CUSTOM_SOUNDS];
+
+	int		isDead;
 } clientInfo_t;
 
 
@@ -450,6 +453,10 @@ typedef struct {
 // occurs, and they will have visible effects for #define STEP_TIME or whatever msec after
 
 #define MAX_PREDICTED_EVENTS	16
+
+//unlagged - optimized prediction
+#define NUM_SAVED_STATES (CMD_BACKUP + 2)
+//unlagged - optimized prediction
  
 typedef struct {
 	int			clientFrame;		// incremented each frame
@@ -645,6 +652,13 @@ typedef struct {
 	char			testModelName[MAX_QPATH];
 	qboolean		testGun;
 
+//unlagged - optimized prediction
+	int			lastPredictedCommand;
+	int			lastServerTime;
+	playerState_t savedPmoveStates[NUM_SAVED_STATES];
+	int			stateHead, stateTail;
+//unlagged - optimized prediction
+
 } cg_t;
 
 
@@ -670,6 +684,11 @@ typedef struct {
 	qhandle_t	blueFlagShader[3];
 	qhandle_t	flagShader[4];
 
+//For Double Domination:
+	qhandle_t	ddPointA;
+	qhandle_t	ddPointB;
+	qhandle_t	ddPointSkin[3]; //0=white,1=red,2=blue
+
 	qhandle_t	flagPoleModel;
 	qhandle_t	flagFlapModel;
 
@@ -1023,6 +1042,35 @@ typedef struct {
 
 	int				levelStartTime;
 
+//Forced FFA
+	int			ffa_gt;
+
+//Elimination
+	int				roundStartTime;	
+	int				roundtime;
+
+//CTF Elimination
+	qboolean			oneway;
+	int				attackingTeam;
+
+//Last Man Standing
+	int				lms_mode;
+
+//instantgib:
+	int				instantgib;
+
+//nexuiz style rocket arena
+	int				rockets;
+
+//Double Domination DD
+	int 				timetaken;
+
+//Domination
+	int domination_points_count;
+	char domination_points_names[MAX_DOMINATION_POINTS][MAX_DOMINATION_POINTS_NAMES];
+	int domination_points_status[MAX_DOMINATION_POINTS];
+
+
 	int				scores1, scores2;		// from configstrings
 	int				redflag, blueflag;		// flag status from configstrings
 	int				flagStatus;
@@ -1068,6 +1116,10 @@ typedef struct {
 	// media
 	cgMedia_t		media;
 
+//unlagged - client options
+	// this will be set to the server's g_delagHitscan
+	int				delagHitscan;
+//unlagged - client options
 } cgs_t;
 
 //==============================================================================
@@ -1152,7 +1204,10 @@ extern	vmCvar_t		cg_teamChatsOnly;
 extern	vmCvar_t		cg_noVoiceChats;
 extern	vmCvar_t		cg_noVoiceText;
 extern  vmCvar_t		cg_scorePlum;
-extern	vmCvar_t		cg_smoothClients;
+//unlagged - smooth clients #2
+// this is done server-side now
+//extern	vmCvar_t		cg_smoothClients;
+//unlagged - smooth clients #2
 extern	vmCvar_t		pmove_fixed;
 extern	vmCvar_t		pmove_msec;
 //extern	vmCvar_t		cg_pmove_fixed;
@@ -1184,6 +1239,29 @@ extern  vmCvar_t		cg_recordSPDemoName;
 extern	vmCvar_t		cg_obeliskRespawnDelay;
 #endif
 
+//unlagged - client options
+extern	vmCvar_t		cg_delag;
+//extern	vmCvar_t		cg_debugDelag;
+//extern	vmCvar_t		cg_drawBBox;
+extern	vmCvar_t		cg_cmdTimeNudge;
+extern	vmCvar_t		sv_fps;
+extern	vmCvar_t		cg_projectileNudge;
+extern	vmCvar_t		cg_optimizePrediction;
+extern	vmCvar_t		cl_timeNudge;
+//extern	vmCvar_t		cg_latentSnaps;
+//extern	vmCvar_t		cg_latentCmds;
+//extern	vmCvar_t		cg_plOut;
+//unlagged - client options
+
+//extra CVARS elimination
+extern	vmCvar_t		cg_alwaysWeaponBar;
+
+//unlagged - cg_unlagged.c
+void CG_PredictWeaponEffects( centity_t *cent );
+//void CG_AddBoundingBox( centity_t *cent );
+qboolean CG_Cvar_ClampInt( const char *name, vmCvar_t *vmCvar, int min, int max );
+//unlagged - cg_unlagged.c
+
 //
 // cg_main.c
 //
@@ -1208,6 +1286,9 @@ void CG_SetScoreSelection(void *menu);
 score_t *CG_GetSelectedScore( void );
 void CG_BuildSpectatorString( void );
 
+//unlagged, sagos modfication
+void SnapVectorTowards( vec3_t v, vec3_t to );
+
 
 //
 // cg_view.c
@@ -1422,6 +1503,9 @@ localEntity_t *CG_MakeExplosion( vec3_t origin, vec3_t dir,
 // cg_snapshot.c
 //
 void CG_ProcessSnapshots( void );
+//unlagged - early transitioning
+void CG_TransitionEntity( centity_t *cent );
+//unlagged - early transitioning
 
 //
 // cg_info.c
