@@ -75,29 +75,45 @@ static void CG_ParseScores( void ) {
 	cg.teamScores[0] = atoi( CG_Argv( 2 ) );
 	cg.teamScores[1] = atoi( CG_Argv( 3 ) );
 
+	cgs.roundStartTime = atoi( CG_Argv( 4 ) );
+
+	//Update thing in lower-right corner
+	if(cgs.gametype == GT_ELIMINATION || cgs.gametype == GT_CTF_ELIMINATION)
+	{
+		cgs.scores1 = cg.teamScores[0];
+		cgs.scores2 = cg.teamScores[1];
+	}
+
 	memset( cg.scores, 0, sizeof( cg.scores ) );
+
+#define NUM_DATA 15
+#define FIRST_DATA 4
+
 	for ( i = 0 ; i < cg.numScores ; i++ ) {
 		//
-		cg.scores[i].client = atoi( CG_Argv( i * 14 + 4 ) );
-		cg.scores[i].score = atoi( CG_Argv( i * 14 + 5 ) );
-		cg.scores[i].ping = atoi( CG_Argv( i * 14 + 6 ) );
-		cg.scores[i].time = atoi( CG_Argv( i * 14 + 7 ) );
-		cg.scores[i].scoreFlags = atoi( CG_Argv( i * 14 + 8 ) );
-		powerups = atoi( CG_Argv( i * 14 + 9 ) );
-		cg.scores[i].accuracy = atoi(CG_Argv(i * 14 + 10));
-		cg.scores[i].impressiveCount = atoi(CG_Argv(i * 14 + 11));
-		cg.scores[i].excellentCount = atoi(CG_Argv(i * 14 + 12));
-		cg.scores[i].guantletCount = atoi(CG_Argv(i * 14 + 13));
-		cg.scores[i].defendCount = atoi(CG_Argv(i * 14 + 14));
-		cg.scores[i].assistCount = atoi(CG_Argv(i * 14 + 15));
-		cg.scores[i].perfect = atoi(CG_Argv(i * 14 + 16));
-		cg.scores[i].captures = atoi(CG_Argv(i * 14 + 17));
+		cg.scores[i].client = atoi( CG_Argv( i * NUM_DATA + FIRST_DATA + 1 ) );
+		cg.scores[i].score = atoi( CG_Argv( i * NUM_DATA + FIRST_DATA + 2 ) );
+		cg.scores[i].ping = atoi( CG_Argv( i * NUM_DATA + FIRST_DATA + 3 ) );
+		cg.scores[i].time = atoi( CG_Argv( i * NUM_DATA + FIRST_DATA + 4 ) );
+		cg.scores[i].scoreFlags = atoi( CG_Argv( i * NUM_DATA + FIRST_DATA + 5 ) );
+		powerups = atoi( CG_Argv( i * NUM_DATA + FIRST_DATA + 6 ) );
+		cg.scores[i].accuracy = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 7));
+		cg.scores[i].impressiveCount = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 8));
+		cg.scores[i].excellentCount = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 9));
+		cg.scores[i].guantletCount = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 10));
+		cg.scores[i].defendCount = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 11));
+		cg.scores[i].assistCount = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 12));
+		cg.scores[i].perfect = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 13));
+		cg.scores[i].captures = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 14));
+		cg.scores[i].isDead = atoi(CG_Argv(i * NUM_DATA + FIRST_DATA + 15));
+		//cgs.roundStartTime = 
 
 		if ( cg.scores[i].client < 0 || cg.scores[i].client >= MAX_CLIENTS ) {
 			cg.scores[i].client = 0;
 		}
 		cgs.clientinfo[ cg.scores[i].client ].score = cg.scores[i].score;
 		cgs.clientinfo[ cg.scores[i].client ].powerups = powerups;
+		cgs.clientinfo[ cg.scores[i].client ].isDead = cg.scores[i].isDead;
 
 		cg.scores[i].team = cgs.clientinfo[cg.scores[i].client].team;
 	}
@@ -107,6 +123,85 @@ static void CG_ParseScores( void ) {
 
 }
 
+/*
+=================
+CG_ParseElimination
+
+=================
+*/
+static void CG_ParseElimination( void ) {
+	if(cgs.gametype == GT_ELIMINATION || cgs.gametype == GT_CTF_ELIMINATION)
+	{
+		cgs.scores1 = atoi( CG_Argv( 1 ) );
+		cgs.scores2 = atoi( CG_Argv( 2 ) );
+	}
+	cgs.roundStartTime = atoi( CG_Argv( 3 ) );
+}
+
+/*
+=================
+CG_ParseDDtimetaken
+
+=================
+*/
+static void CG_ParseDDtimetaken( void ) {
+	cgs.timetaken = atoi( CG_Argv( 1 ) );
+}
+
+/*
+=================
+CG_ParseDomPointNames
+=================
+*/
+
+static void CG_ParseDomPointNames( void ) {
+	int i,j;
+	cgs.domination_points_count = atoi( CG_Argv( 1 ) );
+	if(cgs.domination_points_count>=MAX_DOMINATION_POINTS)
+		cgs.domination_points_count = MAX_DOMINATION_POINTS;
+	for(i = 0;i<cgs.domination_points_count;i++) {
+		Q_strncpyz(cgs.domination_points_names[i],CG_Argv(2)+i*MAX_DOMINATION_POINTS_NAMES,MAX_DOMINATION_POINTS_NAMES-1);
+		for(j=MAX_DOMINATION_POINTS_NAMES-1; cgs.domination_points_names[i][j] < '0' && j>0; j--) {
+			cgs.domination_points_names[i][j] = 0;
+		}
+	}
+}
+
+/*
+=================
+CG_ParseDomScores
+=================
+*/
+
+static void CG_ParseDomStatus( void ) {
+	int i;
+	if( cgs.domination_points_count!=atoi( CG_Argv(1) ) ) {
+		cgs.domination_points_count = 0;
+		return;
+	}
+	for(i = 0;i<cgs.domination_points_count;i++) {
+		cgs.domination_points_status[i] = atoi( CG_Argv(2+i) );
+	}
+}
+
+
+/*
+=================
+CG_ParseAttackingTeam
+
+=================
+*/
+static void CG_ParseAttackingTeam( void ) {
+	int temp;
+	temp = atoi( CG_Argv( 1 ) );
+	if(temp==TEAM_RED)
+		cgs.attackingTeam = TEAM_RED;
+	else if (temp==TEAM_BLUE)
+		cgs.attackingTeam = TEAM_BLUE;
+	else
+		cgs.attackingTeam = TEAM_NONE; //Should never happen.
+}
+
 /*
 =================
 CG_ParseTeamInfo
@@ -147,6 +242,11 @@ void CG_ParseServerinfo( void ) {
 
 	info = CG_ConfigString( CS_SERVERINFO );
 	cgs.gametype = atoi( Info_ValueForKey( info, "g_gametype" ) );
+	//By default do as normal:
+	cgs.ffa_gt = 0;
+	//See if ffa gametype
+	if(cgs.gametype == GT_LMS)	
+		cgs.ffa_gt = 1;
 	trap_Cvar_Set("g_gametype", va("%i", cgs.gametype));
 	cgs.dmflags = atoi( Info_ValueForKey( info, "dmflags" ) );
 	cgs.teamflags = atoi( Info_ValueForKey( info, "teamflags" ) );
@@ -154,12 +254,23 @@ void CG_ParseServerinfo( void ) {
 	cgs.capturelimit = atoi( Info_ValueForKey( info, "capturelimit" ) );
 	cgs.timelimit = atoi( Info_ValueForKey( info, "timelimit" ) );
 	cgs.maxclients = atoi( Info_ValueForKey( info, "sv_maxclients" ) );
+	cgs.roundtime = atoi( Info_ValueForKey( info, "elimination_roundtime" ) );
+	cgs.instantgib = atoi( Info_ValueForKey( info, "g_instantgib" ) );
+	cgs.rockets = atoi( Info_ValueForKey( info, "g_rockets" ) );
+	cgs.lms_mode = atoi( Info_ValueForKey( info, "g_lms_mode" ) );
+	cgs.oneway = atoi( Info_ValueForKey( info, "elimination_ctf_oneway" ) )? qtrue : qfalse;
 	mapname = Info_ValueForKey( info, "mapname" );
 	Com_sprintf( cgs.mapname, sizeof( cgs.mapname ), "maps/%s.bsp", mapname );
 	Q_strncpyz( cgs.redTeam, Info_ValueForKey( info, "g_redTeam" ), sizeof(cgs.redTeam) );
 	trap_Cvar_Set("g_redTeam", cgs.redTeam);
 	Q_strncpyz( cgs.blueTeam, Info_ValueForKey( info, "g_blueTeam" ), sizeof(cgs.blueTeam) );
 	trap_Cvar_Set("g_blueTeam", cgs.blueTeam);
+
+//unlagged - server options
+	// we'll need this for deciding whether or not to predict weapon effects
+	cgs.delagHitscan = atoi( Info_ValueForKey( info, "g_delagHitscan" ) );
+	trap_Cvar_Set("g_delagHitscan", va("%i", cgs.delagHitscan));
+//unlagged - server options
 }
 
 /*
@@ -205,7 +316,7 @@ void CG_SetConfigValues( void ) {
 	cgs.scores1 = atoi( CG_ConfigString( CS_SCORES1 ) );
 	cgs.scores2 = atoi( CG_ConfigString( CS_SCORES2 ) );
 	cgs.levelStartTime = atoi( CG_ConfigString( CS_LEVEL_START_TIME ) );
-	if( cgs.gametype == GT_CTF ) {
+	if( cgs.gametype == GT_CTF || cgs.gametype == GT_CTF_ELIMINATION || cgs.gametype == GT_DOUBLE_D) {
 		s = CG_ConfigString( CS_FLAGSTATUS );
 		cgs.redflag = s[0] - '0';
 		cgs.blueflag = s[1] - '0';
@@ -331,7 +442,7 @@ static void CG_ConfigStringModified( void ) {
 		CG_NewClientInfo( num - CS_PLAYERS );
 		CG_BuildSpectatorString();
 	} else if ( num == CS_FLAGSTATUS ) {
-		if( cgs.gametype == GT_CTF ) {
+		if( cgs.gametype == GT_CTF || cgs.gametype == GT_CTF_ELIMINATION || cgs.gametype == GT_DOUBLE_D) {
 			// format is rb where its red/blue, 0 is at base, 1 is taken, 2 is dropped
 			cgs.redflag = str[0] - '0';
 			cgs.blueflag = str[1] - '0';
@@ -1048,6 +1159,31 @@ static void CG_ServerCommand( void ) {
 		return;
 	}
 
+	if ( !strcmp( cmd, "ddtaken" ) ) {
+		CG_ParseDDtimetaken();
+		return;
+	}
+
+	if ( !strcmp( cmd, "dompointnames" ) ) {
+		CG_ParseDomPointNames();
+		return;
+	}
+
+	if ( !strcmp( cmd, "domStatus" ) ) {
+		CG_ParseDomStatus();
+		return;
+	}
+
+	if ( !strcmp( cmd, "elimination" ) ) {
+		CG_ParseElimination();
+		return;
+	}
+
+	if ( !strcmp( cmd, "attackingteam" ) ) {
+		CG_ParseAttackingTeam();
+		return;
+	}
+
 	if ( !strcmp( cmd, "tinfo" ) ) {
 		CG_ParseTeamInfo();
 		return;
