@@ -134,6 +134,16 @@ void BotPrintTeamGoal(bot_state_t *bs) {
 			BotAI_Print(PRT_MESSAGE, "%s: I'm gonna patrol for %1.0f secs\n", netname, t);
 			break;
 		}
+		case LTG_POINTA:
+		{
+			BotAI_Print(PRT_MESSAGE, "%s: I'm gonna take care of point A for %1.0f secs\n", netname, t);
+			break;
+		}
+		case LTG_POINTB:
+		{
+			BotAI_Print(PRT_MESSAGE, "%s: I'm gonna take care of point B for %1.0f secs\n", netname, t);
+			break;
+		}
 		default:
 		{
 			if (bs->ctfroam_time > FloatTime()) {
@@ -650,6 +660,104 @@ void BotMatch_DefendKeyArea(bot_state_t *bs, bot_match_t *match) {
 #endif //DEBUG
 }
 
+/*
+==================
+BotMatch_TakeA
+For Double Domination
+==================
+*/
+void BotMatch_TakeA(bot_state_t *bs, bot_match_t *match) {
+// 	char itemname[MAX_MESSAGE_SIZE];
+	char netname[MAX_MESSAGE_SIZE];
+	int client;
+
+	if (!TeamPlayIsOn()) return;
+	//if not addressed to this bot
+	if (!BotAddressedToBot(bs, match)) return;
+	//get the match variable
+	/*trap_BotMatchVariable(match, KEYAREA, itemname, sizeof(itemname));
+	//
+	if (!BotGetMessageTeamGoal(bs, itemname, &bs->teamgoal)) {
+		//BotAI_BotInitialChat(bs, "cannotfind", itemname, NULL);
+		//trap_BotEnterChat(bs->cs, bs->client, CHAT_TEAM);
+		return;
+	}*/
+	//
+	trap_BotMatchVariable(match, NETNAME, netname, sizeof(netname));
+	//
+	client = ClientFromName(netname);
+	//the team mate who ordered
+	bs->decisionmaker = client;
+	bs->ordered = qtrue;
+	bs->order_time = FloatTime();
+	//set the time to send a message to the team mates
+	bs->teammessage_time = FloatTime() + 2 * random();
+	//set the ltg type
+	bs->ltgtype = LTG_POINTA;
+	//get the team goal time
+	bs->teamgoal_time = BotGetTime(match);
+	//set the team goal time
+	if (!bs->teamgoal_time) bs->teamgoal_time = FloatTime() + DD_POINTA;
+	//away from defending
+	bs->defendaway_time = 0;
+	//
+	BotSetTeamStatus(bs);
+	// remember last ordered task
+	BotRememberLastOrderedTask(bs);
+#ifdef DEBUG
+	BotPrintTeamGoal(bs);
+#endif //DEBUG
+}
+
+/*
+==================
+BotMatch_TakeB
+For Double Domination
+==================
+*/
+void BotMatch_TakeB(bot_state_t *bs, bot_match_t *match) {
+// 	char itemname[MAX_MESSAGE_SIZE];
+	char netname[MAX_MESSAGE_SIZE];
+	int client;
+
+	if (!TeamPlayIsOn()) return;
+	//if not addressed to this bot
+	if (!BotAddressedToBot(bs, match)) return;
+	//get the match variable
+	/*trap_BotMatchVariable(match, KEYAREA, itemname, sizeof(itemname));
+	//
+	if (!BotGetMessageTeamGoal(bs, itemname, &bs->teamgoal)) {
+		//BotAI_BotInitialChat(bs, "cannotfind", itemname, NULL);
+		//trap_BotEnterChat(bs->cs, bs->client, CHAT_TEAM);
+		//return;
+	}*/
+	//
+	trap_BotMatchVariable(match, NETNAME, netname, sizeof(netname));
+	//
+	client = ClientFromName(netname);
+	//the team mate who ordered
+	bs->decisionmaker = client;
+	bs->ordered = qtrue;
+	bs->order_time = FloatTime();
+	//set the time to send a message to the team mates
+	bs->teammessage_time = FloatTime() + 2 * random();
+	//set the ltg type
+	bs->ltgtype = LTG_POINTB;
+	//get the team goal time
+	bs->teamgoal_time = BotGetTime(match);
+	//set the team goal time
+	if (!bs->teamgoal_time) bs->teamgoal_time = FloatTime() + DD_POINTA;
+	//away from defending
+	bs->defendaway_time = 0;
+	//
+	BotSetTeamStatus(bs);
+	// remember last ordered task
+	BotRememberLastOrderedTask(bs);
+#ifdef DEBUG
+	BotPrintTeamGoal(bs);
+#endif //DEBUG
+}
+
 /*
 ==================
 BotMatch_GetItem
@@ -830,7 +938,7 @@ void BotMatch_GetFlag(bot_state_t *bs, bot_match_t *match) {
 	char netname[MAX_MESSAGE_SIZE];
 	int client;
 
-	if (gametype == GT_CTF) {
+	if (gametype == GT_CTF || gametype == GT_CTF_ELIMINATION) {
 		if (!ctf_redflag.areanum || !ctf_blueflag.areanum)
 			return;
 	}
@@ -860,7 +968,7 @@ void BotMatch_GetFlag(bot_state_t *bs, bot_match_t *match) {
 	//set the team goal time
 	bs->teamgoal_time = FloatTime() + CTF_GETFLAG_TIME;
 	// get an alternate route in ctf
-	if (gametype == GT_CTF) {
+	if (gametype == GT_CTF || gametype == GT_CTF_ELIMINATION) {
 		//get an alternative route goal towards the enemy base
 		BotGetAlternateRouteGoal(bs, BotOppositeTeam(bs));
 	}
@@ -882,7 +990,7 @@ void BotMatch_AttackEnemyBase(bot_state_t *bs, bot_match_t *match) {
 	char netname[MAX_MESSAGE_SIZE];
 	int client;
 
-	if (gametype == GT_CTF) {
+	if (gametype == GT_CTF|| gametype == GT_CTF_ELIMINATION) {
 		BotMatch_GetFlag(bs, match);
 	}
 #ifdef MISSIONPACK
@@ -973,7 +1081,7 @@ void BotMatch_RushBase(bot_state_t *bs, bot_match_t *match) {
 	char netname[MAX_MESSAGE_SIZE];
 	int client;
 
-	if (gametype == GT_CTF) {
+	if (gametype == GT_CTF|| gametype == GT_CTF_ELIMINATION) {
 		if (!ctf_redflag.areanum || !ctf_blueflag.areanum)
 			return;
 	}
@@ -1068,7 +1176,7 @@ void BotMatch_ReturnFlag(bot_state_t *bs, bot_match_t *match) {
 
 	//if not in CTF mode
 	if (
-		gametype != GT_CTF
+		gametype != GT_CTF && gametype != GT_CTF_ELIMINATION
 #ifdef MISSIONPACK
 		&& gametype != GT_1FCTF
 #endif
@@ -1308,7 +1416,7 @@ void BotMatch_StartTeamLeaderShip(bot_state_t *bs, bot_match_t *match) {
 		//get the team mate that will be the team leader
 		trap_BotMatchVariable(match, NETNAME, teammate, sizeof(teammate));
 		strncpy(bs->teamleader, teammate, sizeof(bs->teamleader));
-		bs->teamleader[sizeof(bs->teamleader)] = '\0';
+		bs->teamleader[sizeof(bs->teamleader)-1] = '\0';
 	}
 	//chats for someone else
 	else {
@@ -1448,6 +1556,16 @@ void BotMatch_WhatAreYouDoing(bot_state_t *bs, bot_match_t *match) {
 			break;
 		}
 #endif
+		case LTG_POINTA:
+		{
+			BotAI_BotInitialChat(bs, "dd_pointa", NULL);
+			break;
+		}
+		case LTG_POINTB:
+		{
+			BotAI_BotInitialChat(bs, "dd_pointb", NULL);
+			break;
+		}
 		default:
 		{
 			BotAI_BotInitialChat(bs, "roaming", NULL);
@@ -1567,7 +1685,7 @@ void BotMatch_WhereAreYou(bot_state_t *bs, bot_match_t *match) {
 		}
 	}
 	if (bestitem != -1) {
-		if (gametype == GT_CTF
+		if (gametype == GT_CTF || gametype == GT_CTF_ELIMINATION
 #ifdef MISSIONPACK
 			|| gametype == GT_1FCTF
 #endif
@@ -1724,7 +1842,7 @@ void BotMatch_CTF(bot_state_t *bs, bot_match_t *match) {
 
 	char flag[128], netname[MAX_NETNAME];
 
-	if (gametype == GT_CTF) {
+	if (gametype == GT_CTF || gametype == GT_CTF_ELIMINATION) {
 		trap_BotMatchVariable(match, FLAG, flag, sizeof(flag));
 		if (match->subtype & ST_GOTFLAG) {
 			if (!Q_stricmp(flag, "red")) {
@@ -1802,8 +1920,9 @@ int BotMatchMessage(bot_state_t *bs, char *message) {
 	match.type = 0;
 	//if it is an unknown message
 	if (!trap_BotFindMatch(message, &match, MTCONTEXT_MISC
-											|MTCONTEXT_INITIALTEAMCHAT
-											|MTCONTEXT_CTF)) {
+	|MTCONTEXT_INITIALTEAMCHAT
+	|MTCONTEXT_CTF
+	|MTCONTEXT_DD)) {
 		return qfalse;
 	}
 	//react to the found message
@@ -1982,6 +2101,16 @@ int BotMatchMessage(bot_state_t *bs, char *message) {
 			BotMatch_Suicide(bs, &match);
 			break;
 		}
+		case MSG_TAKEA:
+		{
+			BotMatch_TakeA(bs, &match);
+			break;
+		}
+		case MSG_TAKEB:
+		{
+			BotMatch_TakeB(bs, &match);
+			break;
+		}
 		default:
 		{
 			BotAI_Print(PRT_MESSAGE, "unknown match type\n");
