@@ -916,6 +916,161 @@ Only in One Flag CTF games
 	},
 #endif
 
+/*QUAKED team_DD_point
+Only in DD games
+*/
+	{
+		"team_DD_pointAblue",
+		NULL,
+        { "models/dpoints/a_blue.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/icona_blu",
+/* pickup */	"Point A (Blue)",
+		0,
+		IT_TEAM,
+		DD_POINTABLUE,
+/* precache */ "",
+/* sounds */ ""
+	},
+
+/*QUAKED team_DD_point
+Only in DD games
+*/
+	{
+		"team_DD_pointBblue",
+		NULL,
+        { "models/dpoints/b_blue.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/iconb_blu",
+/* pickup */	"Point B (Blue)",
+		0,
+		IT_TEAM,
+		DD_POINTBBLUE,
+/* precache */ "",
+/* sounds */ ""
+	},
+
+/*QUAKED team_DD_point
+Only in DD games
+*/
+	{
+		"team_DD_pointAred",
+		NULL,
+        { "models/dpoints/a_red.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/icona_red",
+/* pickup */	"Point A (Red)",
+		0,
+		IT_TEAM,
+		DD_POINTARED,
+/* precache */ "",
+/* sounds */ ""
+	},
+
+/*QUAKED team_DD_point
+Only in DD games
+*/
+	{
+		"team_DD_pointBred",
+		NULL,
+        { "models/dpoints/b_red.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/iconb_red",
+/* pickup */	"Point B (Red)",
+		0,
+		IT_TEAM,
+		DD_POINTBRED,
+/* precache */ "",
+/* sounds */ ""
+	},
+
+/*QUAKED team_DD_point
+Only in DD games
+*/
+	{
+		"team_DD_pointAwhite",
+		NULL,
+        { "models/dpoints/a_white.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/icona_white",
+/* pickup */	"Point A (White)",
+		0,
+		IT_TEAM,
+		DD_POINTAWHITE,
+/* precache */ "",
+/* sounds */ ""
+	},
+
+/*QUAKED team_DD_point
+Only in DD games
+*/
+	{
+		"team_DD_pointBwhite",
+		NULL,
+        { "models/dpoints/b_white.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/iconb_white",
+/* pickup */	"Point B (White)",
+		0,
+		IT_TEAM,
+		DD_POINTBWHITE,
+/* precache */ "",
+/* sounds */ ""
+	},
+
+//Now things for standard domination:
+
+/*QUAKED 
+Only in Domination games
+*/
+	{
+		"team_dom_pointWhite",
+		NULL,
+        { "models/dpoints/a_white.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/icona_white",
+/* pickup */	"Neutral domination point",
+		0,
+		IT_TEAM,
+		DOM_POINTWHITE,
+/* precache */ "",
+/* sounds */ ""
+	},
+
+/*QUAKED 
+Only in Domination games
+*/
+	{
+		"team_dom_pointRed",
+		NULL,
+        { "models/dpoints/a_red.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/icona_red",
+/* pickup */	"Red domination point",
+		0,
+		IT_TEAM,
+		DOM_POINTRED,
+/* precache */ "",
+/* sounds */ ""
+	},
+
+/*QUAKED 
+Only in Domination games
+*/
+	{
+		"team_dom_pointBlue",
+		NULL,
+        { "models/dpoints/a_blue.md3",
+		NULL, NULL, NULL },
+/* icon */		"icons/icona_blue",
+/* pickup */	"Blue domination point",
+		0,
+		IT_TEAM,
+		DOM_POINTBLUE,
+/* precache */ "",
+/* sounds */ ""
+	},
+
 	// end of list marker
 	{NULL}
 };
@@ -1108,6 +1263,11 @@ qboolean BG_CanItemBeGrabbed( int gametype, const entityState_t *ent, const play
 
 #ifdef MISSIONPACK
 	case IT_PERSISTANT_POWERUP:
+
+		//In Double D we don't want persistant Powerups (or maybe, can be discussed)
+		if(gametype == GT_DOUBLE_D)
+			return qfalse;
+
 		// can only hold one item at a time
 		if ( ps->stats[STAT_PERSISTANT_POWERUP] ) {
 			return qfalse;
@@ -1142,7 +1302,7 @@ qboolean BG_CanItemBeGrabbed( int gametype, const entityState_t *ent, const play
 			}
 		}
 #endif
-		if( gametype == GT_CTF ) {
+		if( gametype == GT_CTF || gametype == GT_CTF_ELIMINATION) {
 			// ent->modelindex2 is non-zero on items if they are dropped
 			// we need to know this because we can pick up our dropped flag (and return it)
 			// but we can't pick up our flag at base
@@ -1159,6 +1319,24 @@ qboolean BG_CanItemBeGrabbed( int gametype, const entityState_t *ent, const play
 			}
 		}
 
+		if( gametype == GT_DOUBLE_D) {
+			//We can touch both flags
+			if(item->giTag == PW_BLUEFLAG || item->giTag == PW_REDFLAG)
+				return qtrue;
+		}
+
+		if( gametype == GT_DOMINATION ) {
+			if(item->giTag == DOM_POINTWHITE)
+				return qtrue;
+			if (ps->persistant[PERS_TEAM] == TEAM_RED) {
+				if(item->giTag == DOM_POINTBLUE)
+					return qtrue;
+			} else if (ps->persistant[PERS_TEAM] == TEAM_BLUE) {
+				if(item->giTag == DOM_POINTRED)
+					return qtrue;
+			}
+		}
+
 #ifdef MISSIONPACK
 		if( gametype == GT_HARVESTER ) {
 			return qtrue;
