@@ -55,6 +55,14 @@ void SP_info_player_start(gentity_t *ent) {
 	SP_info_player_deathmatch( ent );
 }
 
+//One for Double_D
+void SP_info_player_dd(gentity_t *ent) {
+}
+
+//One for Standard Domination, not really a player spawn point
+void SP_domination_point(gentity_t *ent) {
+}
+
 /*QUAKED info_player_intermission (1 0 1) (-16 -16 -24) (16 16 32)
 The intermission will be viewed from this point.  Target an info_notnull for the view direction.
 */
@@ -306,12 +314,24 @@ SelectSpectatorSpawnPoint
 ============
 */
 gentity_t *SelectSpectatorSpawnPoint( vec3_t origin, vec3_t angles ) {
+	//gentity_t	*spot;
+
 	FindIntermissionPoint();
 
 	VectorCopy( level.intermission_origin, origin );
 	VectorCopy( level.intermission_angle, angles );
 
-	return NULL;
+
+
+	//for some reason we need to return an specific point in elimination (this might not be neccecary anymore but to be sure...)
+	//if(g_gametype.integer == GT_ELIMINATION)
+	//	return SelectSpawnPoint( vec3_origin, origin, angles );
+
+	//VectorCopy (origin,spot->s.origin);
+	//spot->s.origin[2] += 9;
+	//VectorCopy (angles, spot->s.angles);
+
+	return NULL; //spot;
 }
 
 /*
@@ -499,12 +519,72 @@ respawn
 void respawn( gentity_t *ent ) {
 	gentity_t	*tent;
 
-	CopyToBodyQue (ent);
-	ClientSpawn(ent);
+	if((g_gametype.integer!=GT_ELIMINATION && g_gametype.integer!=GT_CTF_ELIMINATION && g_gametype.integer !=GT_LMS) && !ent->client->isEliminated)
+	{
+		ent->client->isEliminated = qtrue; //must not be true in warmup
+		//Tried moving CopyToBodyQue
+	}
+        CopyToBodyQue (ent); //Unlinks ent
 
-	// add a teleportation effect
-	tent = G_TempEntity( ent->client->ps.origin, EV_PLAYER_TELEPORT_IN );
-	tent->s.clientNum = ent->s.clientNum;
+	if(g_gametype.integer==GT_LMS) {
+		if(ent->client->pers.livesLeft>0)
+		{
+			//ent->client->pers.livesLeft--; Coutned down somewhere else
+			ent->client->isEliminated = qfalse;
+		}
+		else //We have used all our lives
+		{
+			if( ent->client->isEliminated!=qtrue) {
+				ent->client->isEliminated = qtrue;
+				if((g_lms_mode.integer == 2 || g_lms_mode.integer == 3) && level.roundNumber == level.roundNumberStarted)
+					LMSpoint();	
+                                //Sago: This is really bad
+                                //TODO: Try not to make people spectators here
+				ent->client->sess.spectatorState = PM_SPECTATOR;
+                                //We have to force spawn imidiantly to precent lag.
+                                ClientSpawn(ent);
+			}
+			return;
+		}
+	}
+
+	if((g_gametype.integer==GT_ELIMINATION || g_gametype.integer==GT_CTF_ELIMINATION || g_gametype.integer==GT_LMS) 
+			&& ent->client->ps.pm_type == PM_SPECTATOR && ent->client->ps.stats[STAT_HEALTH] > 0)
+		return;
+		ClientSpawn(ent);
+
+		// add a teleportation effect
+		if(g_gametype.integer!=GT_ELIMINATION && g_gametype.integer!=GT_CTF_ELIMINATION && g_gametype.integer!=GT_LMS)
+		{	
+			tent = G_TempEntity( ent->client->ps.origin, EV_PLAYER_TELEPORT_IN );
+			tent->s.clientNum = ent->s.clientNum;
+		}
+}
+
+/*
+================
+respawnRound
+================
+*/
+void respawnRound( gentity_t *ent ) {
+	gentity_t	*tent;
+
+	//if(g_gametype.integer!=GT_ELIMINATION || !ent->client->isEliminated)
+	//{
+	//	ent->client->isEliminated  = qtrue;
+		//CopyToBodyQue (ent);
+	//}
+
+	//if(g_gametype.integer==GT_ELIMINATION && ent->client->ps.pm_type == PM_SPECTATOR && ent->client->ps.stats[STAT_HEALTH] > 0)
+	//	return;
+		ClientSpawn(ent);
+
+		// add a teleportation effect
+		if(g_gametype.integer!=GT_ELIMINATION && g_gametype.integer!=GT_CTF_ELIMINATION && g_gametype.integer!=GT_LMS)
+		{	
+			tent = G_TempEntity( ent->client->ps.origin, EV_PLAYER_TELEPORT_IN );
+			tent->s.clientNum = ent->s.clientNum;
+		}
 }
 
 /*
@@ -533,6 +613,226 @@ team_t TeamCount( int ignoreClientNum, int team ) {
 	return count;
 }
 
+/*
+================
+TeamLivingCount
+
+Returns number of living players on a team
+================
+*/
+team_t TeamLivingCount( int ignoreClientNum, int team ) {
+	int		i;
+	int		count = 0;
+	qboolean	LMS = (g_gametype.integer==GT_LMS);
+
+	for ( i = 0 ; i < level.maxclients ; i++ ) {
+		if ( i == ignoreClientNum ) {
+			continue;
+		}
+		if ( level.clients[i].pers.connected == CON_DISCONNECTED ) {
+			continue;
+		}
+		//crash if g_gametype.integer is used here, why?
+		if ( level.clients[i].sess.sessionTeam == team && (level.clients[i].ps.stats[STAT_HEALTH]>0 || LMS) && !(level.clients[i].isEliminated)) {
+			count++;
+		}
+	}
+
+	return count;
+}
+
+/*
+================
+TeamHealthCount
+
+Count total number of healthpoints on teh teams used for draws in Elimination
+================
+*/
+
+team_t TeamHealthCount(int ignoreClientNum, int team ) {
+	int 		i;
+	int 		count = 0;
+
+	for ( i = 0 ; i < level.maxclients ; i++ ) {
+		if ( i == ignoreClientNum ) {
+			continue;
+		}
+		if ( level.clients[i].pers.connected == CON_DISCONNECTED ) {
+			continue;
+		}
+		//only count clients with positive health
+		if ( level.clients[i].sess.sessionTeam == team && (level.clients[i].ps.stats[STAT_HEALTH]>0)&& !(level.clients[i].isEliminated)) {
+			count+=level.clients[i].ps.stats[STAT_HEALTH];
+		}
+	}
+
+	return count;
+}
+
+
+/*
+================
+RespawnAll
+
+Forces all clients to respawn.
+================
+*/
+
+void RespawnAll(void)
+{
+	int i;
+	gentity_t	*client;
+	for(i=0;i<level.maxclients;i++)
+	{
+		if ( level.clients[i].pers.connected == CON_DISCONNECTED ) {
+			continue;
+		}
+
+		if ( level.clients[i].sess.sessionTeam == TEAM_SPECTATOR ) {
+			continue;
+		}
+		client = g_entities + i;
+		client->client->ps.pm_type = PM_NORMAL;
+		client->client->pers.livesLeft = g_lms_lives.integer;
+		respawnRound(client);
+	}
+	return;
+}
+
+/*
+================
+RespawnDead
+
+Forces all *DEAD* clients to respawn.
+================
+*/
+
+void RespawnDead(void)
+{
+	int i;
+	gentity_t	*client;
+	for(i=0;i<level.maxclients;i++)
+	{
+		
+		if ( level.clients[i].pers.connected == CON_DISCONNECTED ) {
+			continue;
+		}
+                client = g_entities + i;
+                client->client->pers.livesLeft = g_lms_lives.integer-1;
+		if ( level.clients[i].isEliminated == qfalse ){
+			continue;
+		}
+		if ( level.clients[i].sess.sessionTeam == TEAM_SPECTATOR ) {
+			continue;
+		}
+		
+		client->client->pers.livesLeft = g_lms_lives.integer;
+                
+		respawnRound(client);
+	}
+	return;
+}
+
+/*
+================
+DisableWeapons
+
+disables all weapons
+================
+*/
+
+void DisableWeapons(void)
+{
+	int i;
+	gentity_t	*client;
+	for(i=0;i<level.maxclients;i++)
+	{
+		if ( level.clients[i].pers.connected == CON_DISCONNECTED ) {
+			continue;
+		}
+
+		if ( level.clients[i].sess.sessionTeam == TEAM_SPECTATOR ) {
+			continue;
+		}
+		client = g_entities + i;
+		client->client->ps.pm_flags |= PMF_ELIMWARMUP;
+	}
+	return;
+}
+
+/*
+================
+EnableWeapons
+
+enables all weapons
+================
+*/
+
+void EnableWeapons(void)
+{
+	int i;
+	gentity_t	*client;
+	for(i=0;i<level.maxclients;i++)
+	{
+		if ( level.clients[i].pers.connected == CON_DISCONNECTED ) {
+			continue;
+		}
+
+		if ( level.clients[i].sess.sessionTeam == TEAM_SPECTATOR ) {
+			continue;
+		}
+
+		/*if ( level.clients[i].isEliminated == qtrue ){
+			continue;
+		}*/
+
+		client = g_entities + i;
+		client->client->ps.pm_flags &= ~PMF_ELIMWARMUP;
+	}
+	return;
+}
+
+/*
+================
+LMSpoint
+
+Gives a point to the lucky survivor
+================
+*/
+
+void LMSpoint(void)
+{
+	int i;
+	gentity_t	*client;
+	for(i=0;i<level.maxclients;i++)
+	{
+		if ( level.clients[i].pers.connected == CON_DISCONNECTED ) {
+			continue;
+		}
+
+		if ( level.clients[i].sess.sessionTeam == TEAM_SPECTATOR ) {
+			continue;
+		}
+
+		if ( level.clients[i].isEliminated == qtrue ){
+			continue;
+		}
+		
+		client = g_entities + i;
+		/*
+		Not good in mode 2 & 3
+		if ( client->health <= 0 ){
+			continue;
+		}
+		*/
+	
+		client->client->ps.persistant[PERS_SCORE] += 1;
+	}
+	
+	CalculateRanks();
+	return;
+}
+
 /*
 ================
 TeamLeader
@@ -629,7 +929,7 @@ static void ClientCleanName( const char *in, char *out, int outSize ) {
 		}
 
 		// don't allow leading spaces
-		if( !*p && ch == ' ' ) {
+		if( colorlessLen == 0 && ch == ' ' ) {
 			continue;
 		}
 
@@ -658,11 +958,15 @@ static void ClientCleanName( const char *in, char *out, int outSize ) {
 		}
 
 		// don't allow too many consecutive spaces
-		if( ch == ' ' ) {
+		// don't count spaces in colorlessLen
+                if( ch == ' ' ) {
 			spaces++;
 			if( spaces > 3 ) {
 				continue;
 			}
+                        *out++ = ch;
+			len++;
+			continue;
 		}
 		else {
 			spaces = 0;
@@ -734,15 +1038,59 @@ void ClientUserinfoChanged( int clientNum ) {
 		client->pers.predictItemPickup = qtrue;
 	}
 
+//unlagged - client options
+	// see if the player has opted out
+	s = Info_ValueForKey( userinfo, "cg_delag" );
+	if ( !atoi( s ) ) {
+		client->pers.delag = 0;
+	} else {
+		client->pers.delag = atoi( s );
+	}
+
+	// see if the player is nudging his shots
+	s = Info_ValueForKey( userinfo, "cg_cmdTimeNudge" );
+	client->pers.cmdTimeNudge = atoi( s );
+
+	// see if the player wants to debug the backward reconciliation
+	/*s = Info_ValueForKey( userinfo, "cg_debugDelag" );
+	if ( !atoi( s ) ) {
+		client->pers.debugDelag = qfalse;
+	}
+	else {
+		client->pers.debugDelag = qtrue;
+	}*/
+
+	// see if the player is simulating incoming latency
+	//s = Info_ValueForKey( userinfo, "cg_latentSnaps" );
+	//client->pers.latentSnaps = atoi( s );
+
+	// see if the player is simulating outgoing latency
+	//s = Info_ValueForKey( userinfo, "cg_latentCmds" );
+	//client->pers.latentCmds = atoi( s );
+
+	// see if the player is simulating outgoing packet loss
+	//s = Info_ValueForKey( userinfo, "cg_plOut" );
+	//client->pers.plOut = atoi( s );
+//unlagged - client options
+
 	// set name
 	Q_strncpyz ( oldname, client->pers.netname, sizeof( oldname ) );
 	s = Info_ValueForKey (userinfo, "name");
 	ClientCleanName( s, client->pers.netname, sizeof(client->pers.netname) );
 
-	if ( client->sess.sessionTeam == TEAM_SPECTATOR ) {
-		if ( client->sess.spectatorState == SPECTATOR_SCOREBOARD ) {
-			Q_strncpyz( client->pers.netname, "scoreboard", sizeof(client->pers.netname) );
-		}
+	// N_G: this condition makes no sense to me and I'm not going to
+	// try finding out what it means, I've added parentheses according to
+	// evaluation rules of the original code so grab a
+	// parentheses pairing highlighting text editor and see for yourself
+	// if you got it right
+	//Sago: One redundant check and CTF Elim and LMS was missing. Not an important function and I might never have noticed, should properly be ||
+	if ( ( ( client->sess.sessionTeam == TEAM_SPECTATOR ) ||
+		( ( ( client->isEliminated ) /*||
+		( client->ps.pm_type == PM_SPECTATOR )*/ ) &&   //Sago: If this is true client.isEliminated or TEAM_SPECTATOR is true to and this is redundant
+		( g_gametype.integer == GT_ELIMINATION || g_gametype.integer == GT_CTF_ELIMINATION || g_gametype.integer == GT_LMS) ) ) &&
+		( client->sess.spectatorState == SPECTATOR_SCOREBOARD ) ) {
+
+		Q_strncpyz( client->pers.netname, "scoreboard", sizeof(client->pers.netname) );
 	}
 
 	if ( client->pers.connected == CON_CONNECTED ) {
@@ -773,7 +1121,7 @@ void ClientUserinfoChanged( int clientNum ) {
 	client->ps.stats[STAT_MAX_HEALTH] = client->pers.maxHealth;
 
 	// set model
-	if( g_gametype.integer >= GT_TEAM ) {
+	if( g_gametype.integer >= GT_TEAM && g_ffa_gt==0) {
 		Q_strncpyz( model, Info_ValueForKey (userinfo, "team_model"), sizeof( model ) );
 		Q_strncpyz( headModel, Info_ValueForKey (userinfo, "team_headmodel"), sizeof( headModel ) );
 	} else {
@@ -782,7 +1130,7 @@ void ClientUserinfoChanged( int clientNum ) {
 	}
 
 	// bots set their team a few frames later
-	if (g_gametype.integer >= GT_TEAM && g_entities[clientNum].r.svFlags & SVF_BOT) {
+	if (g_gametype.integer >= GT_TEAM && g_ffa_gt==0 && g_entities[clientNum].r.svFlags & SVF_BOT) {
 		s = Info_ValueForKey( userinfo, "team" );
 		if ( !Q_stricmp( s, "red" ) || !Q_stricmp( s, "r" ) ) {
 			team = TEAM_RED;
@@ -819,7 +1167,7 @@ void ClientUserinfoChanged( int clientNum ) {
 */
 
 #ifdef MISSIONPACK
-	if (g_gametype.integer >= GT_TEAM) {
+	if (g_gametype.integer >= GT_TEAM && g_ffa_gt!=1) {
 		client->pers.teamInfo = qtrue;
 	} else {
 		s = Info_ValueForKey( userinfo, "teamoverlay" );
@@ -978,6 +1326,17 @@ char *ClientConnect( int clientNum, qboolean firstTime, qboolean isBot ) {
 //	if ( !client->areabits )
 //		client->areabits = G_Alloc( (trap_AAS_PointReachabilityAreaIndex( NULL ) + 7) / 8 );
 
+//Sago: Changed the message
+//unlagged - backward reconciliation #5
+	// announce it
+	if ( g_delagHitscan.integer ) {
+		trap_SendServerCommand( clientNum, "print \"Full lag compensation is ON!\n\"" );
+	}
+	else {
+		trap_SendServerCommand( clientNum, "print \"Full lag compensation is OFF!\n\"" );
+	}
+//unlagged - backward reconciliation #5
+
 	return NULL;
 }
 
@@ -1012,6 +1371,13 @@ void ClientBegin( int clientNum ) {
 	client->pers.enterTime = level.time;
 	client->pers.teamState.state = TEAM_BEGIN;
 
+	//Elimination:
+	client->pers.roundReached = 0; //We will spawn in next round
+	if(g_gametype.integer == GT_LMS) {
+		client->isEliminated = qtrue; //So player does not give a point in gamemode 2 and 3
+		//trap_SendServerCommand( -1, va("print \"%s" S_COLOR_WHITE " will start dead\n\"", client->pers.netname) );
+	}
+
 	// save eflags around this, because changing teams will
 	// cause this to happen with a valid entity, and we
 	// want to make sure the teleport bit is set right
@@ -1024,7 +1390,13 @@ void ClientBegin( int clientNum ) {
 	// locate ent at a spawn point
 	ClientSpawn( ent );
 
-	if ( client->sess.sessionTeam != TEAM_SPECTATOR ) {
+	// N_G: I'm really sure line 1333 is not what you meant
+	if( ( client->sess.sessionTeam != TEAM_SPECTATOR ) &&
+		( ( !( client->isEliminated ) /*&&
+		( ( !client->ps.pm_type ) == PM_SPECTATOR ) */ ) || //Sago: Yes, it made no sense 
+		( ( g_gametype.integer != GT_ELIMINATION ) &&
+		( g_gametype.integer != GT_CTF_ELIMINATION ) &&
+		( g_gametype.integer != GT_LMS ) ) ) ) {
 		// send event
 		tent = G_TempEntity( ent->client->ps.origin, EV_PLAYER_TELEPORT_IN );
 		tent->s.clientNum = ent->s.clientNum;
@@ -1035,6 +1407,12 @@ void ClientBegin( int clientNum ) {
 	}
 	G_LogPrintf( "ClientBegin: %i\n", clientNum );
 
+	//Send domination point names:
+	if(g_gametype.integer == GT_DOMINATION) {
+		DominationPointNamesMessage(ent);
+		DominationPointStatusMessage(ent);
+	}
+
 	// count current clients and rank for scoreboard
 	CalculateRanks();
 }
@@ -1067,13 +1445,72 @@ void ClientSpawn(gentity_t *ent) {
 	index = ent - g_entities;
 	client = ent->client;
 
+	//In Elimination the player should not spawn if he have already spawned in the round (but not for spectators)
+	// N_G: You've obviously wanted something ELSE
+	//Sago: Yes, the !level.intermissiontime is currently redundant but it might still be the bast place to make the test, CheckElimination in g_main makes sure the next if will fail and the rest of the things this block does might not affect if in Intermission (I'll just test that)
+	if( 
+	( 
+		( 
+			g_gametype.integer == GT_ELIMINATION ||
+			g_gametype.integer == GT_CTF_ELIMINATION || (g_gametype.integer == GT_LMS && client->isEliminated)) &&
+			(!level.intermissiontime || level.warmupTime != 0)
+		) &&
+		( client->sess.sessionTeam != TEAM_SPECTATOR ) 
+	)
+	{
+		// N_G: Another condition that makes no sense to me, see for
+		// yourself if you really meant this
+		// Sago: I beleive the TeamCount is to make sure people can join even if the game can't start
+		if( ( level.roundNumber == level.roundNumberStarted ) ||
+			( (level.time < level.roundStartTime - g_elimination_activewarmup.integer*1000 ) &&
+			TeamCount( -1, TEAM_BLUE ) &&
+			TeamCount( -1, TEAM_RED )  ) )
+		{	
+			client->sess.spectatorState = SPECTATOR_FREE;
+			client->isEliminated = qtrue;
+			client->ps.pm_type = PM_SPECTATOR;
+			return;
+		}
+		else
+		{
+			client->pers.roundReached = level.roundNumber+1;
+			client->sess.spectatorState = SPECTATOR_NOT;
+			client->ps.pm_type = PM_NORMAL;
+			client->isEliminated = qfalse;
+		}
+	}
+
+	if(g_gametype.integer == GT_LMS && client->sess.sessionTeam != TEAM_SPECTATOR && (!level.intermissiontime || level.warmupTime != 0))
+	{
+		if(level.roundNumber==level.roundNumberStarted /*|| level.time<level.roundStartTime-g_elimination_activewarmup.integer*1000*/ && 1>client->pers.livesLeft)
+		{	
+			client->sess.spectatorState = SPECTATOR_FREE;
+			if( ent->client->isEliminated!=qtrue) {
+				client->isEliminated = qtrue;
+				if((g_lms_mode.integer == 2 || g_lms_mode.integer == 3) && level.roundNumber == level.roundNumberStarted)
+					LMSpoint();
+			}
+			client->ps.pm_type = PM_SPECTATOR;
+			return;
+		}
+		
+		client->sess.spectatorState = SPECTATOR_NOT;
+		client->ps.pm_type = PM_NORMAL;
+		client->isEliminated = qfalse;
+		if(client->pers.livesLeft>0)
+			client->pers.livesLeft--;
+	}
+
 	// find a spawn point
 	// do it before setting health back up, so farthest
 	// ranging doesn't count this client
-	if ( client->sess.sessionTeam == TEAM_SPECTATOR ) {
-		spawnPoint = SelectSpectatorSpawnPoint ( 
-						spawn_origin, spawn_angles);
-	} else if (g_gametype.integer >= GT_CTF ) {
+	if ((client->sess.sessionTeam == TEAM_SPECTATOR) 
+			|| ( (client->ps.pm_type == PM_SPECTATOR || client->isEliminated )  && (g_gametype.integer == GT_ELIMINATION || g_gametype.integer == GT_CTF_ELIMINATION) ) ) {
+		spawnPoint = SelectSpectatorSpawnPoint ( spawn_origin, spawn_angles);
+	} else if (g_gametype.integer == GT_DOUBLE_D) {
+		//Double Domination uses special spawn points:
+		spawnPoint = SelectDoubleDominationSpawnPoint (spawn_origin, spawn_angles);
+	} else if (g_gametype.integer >= GT_CTF && g_ffa_gt==0) {
 		// all base oriented team games use the CTF spawn points
 		spawnPoint = SelectCTFSpawnPoint ( 
 						client->sess.sessionTeam, 
@@ -1116,6 +1553,13 @@ void ClientSpawn(gentity_t *ent) {
 	flags = ent->client->ps.eFlags & (EF_TELEPORT_BIT | EF_VOTED | EF_TEAMVOTED);
 	flags ^= EF_TELEPORT_BIT;
 
+//unlagged - backward reconciliation #3
+	// we don't want players being backward-reconciled to the place they died
+	G_ResetHistory( ent );
+	// and this is as good a time as any to clear the saved state
+	ent->client->saved.leveltime = 0;
+//unlagged - backward reconciliation #3
+
 	// clear everything but the persistant data
 
 	saved = client->pers;
@@ -1129,7 +1573,7 @@ void ClientSpawn(gentity_t *ent) {
 	}
 	eventSequence = client->ps.eventSequence;
 
-	memset (client, 0, sizeof(*client)); // bk FIXME: Com_Memset?
+	Com_Memset (client, 0, sizeof(*client));
 
 	client->pers = saved;
 	client->sess = savedSess;
@@ -1170,12 +1614,17 @@ void ClientSpawn(gentity_t *ent) {
 	ent->waterlevel = 0;
 	ent->watertype = 0;
 	ent->flags = 0;
+        
+        //Sago: No one has hit the client yet!
+        client->lastSentFlying = -1;
 	
 	VectorCopy (playerMins, ent->r.mins);
 	VectorCopy (playerMaxs, ent->r.maxs);
 
 	client->ps.clientNum = index;
 
+if(g_gametype.integer != GT_ELIMINATION && g_gametype.integer != GT_CTF_ELIMINATION && g_gametype.integer != GT_LMS)
+{
 	client->ps.stats[STAT_WEAPONS] = ( 1 << WP_MACHINEGUN );
 	if ( g_gametype.integer == GT_TEAM ) {
 		client->ps.ammo[WP_MACHINEGUN] = 50;
@@ -1189,18 +1638,102 @@ void ClientSpawn(gentity_t *ent) {
 
 	// health will count down towards max_health
 	ent->health = client->ps.stats[STAT_HEALTH] = client->ps.stats[STAT_MAX_HEALTH] + 25;
+}
+else
+{
+	client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_GAUNTLET );
+	client->ps.ammo[WP_GAUNTLET] = -1;
+	client->ps.ammo[WP_GRAPPLING_HOOK] = -1;
+	if (g_elimination_machinegun.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_MACHINEGUN );
+		client->ps.ammo[WP_MACHINEGUN] = g_elimination_machinegun.integer;
+	}
+	if (g_elimination_shotgun.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_SHOTGUN );
+		client->ps.ammo[WP_SHOTGUN] = g_elimination_shotgun.integer;
+	}
+	if (g_elimination_grenade.integer > 0) {	
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_GRENADE_LAUNCHER );
+		client->ps.ammo[WP_GRENADE_LAUNCHER] = g_elimination_grenade.integer;
+	}
+	if (g_elimination_rocket.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_ROCKET_LAUNCHER );
+		client->ps.ammo[WP_ROCKET_LAUNCHER] = g_elimination_rocket.integer;
+	}
+	if (g_elimination_lightning.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_LIGHTNING );
+		client->ps.ammo[WP_LIGHTNING] = g_elimination_lightning.integer;
+	}
+	if (g_elimination_railgun.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_RAILGUN );
+		client->ps.ammo[WP_RAILGUN] = g_elimination_railgun.integer;
+	}
+	if (g_elimination_plasmagun.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_PLASMAGUN );
+		client->ps.ammo[WP_PLASMAGUN] = g_elimination_plasmagun.integer;
+	}
+	if (g_elimination_bfg.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_BFG );
+		client->ps.ammo[WP_BFG] = g_elimination_bfg.integer;
+	}
+#ifdef MISSIONPACK
+	if (g_elimination_nail.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_NAILGUN );
+		client->ps.ammo[WP_NAILGUN] = g_elimination_nail.integer;
+	}
+	if (g_elimination_mine.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_PROX_LAUNCHER );
+		client->ps.ammo[WP_PROX_LAUNCHER] = g_elimination_mine.integer;
+	}
+	if (g_elimination_chain.integer > 0) {
+		client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_CHAINGUN );
+		client->ps.ammo[WP_CHAINGUN] = g_elimination_chain.integer;
+	}
+#endif
+	
+	ent->health = client->ps.stats[STAT_ARMOR] = g_elimination_startArmor.integer; //client->ps.stats[STAT_MAX_HEALTH]*2;
+	ent->health = client->ps.stats[STAT_HEALTH] = g_elimination_startHealth.integer; //client->ps.stats[STAT_MAX_HEALTH]*2;	
+
+	
+	//	ent->health = client->ps.stats[STAT_HEALTH] = 0;
+}
+	//Instantgib mode, replace weapons with rail (and maybe gauntlet)
+	if(g_instantgib.integer)
+	{
+		client->ps.stats[STAT_WEAPONS] = ( 1 << WP_RAILGUN );
+		client->ps.ammo[WP_RAILGUN] = 999; //Don't display any ammo
+		if(g_instantgib.integer>1)
+		{
+			 client->ps.stats[STAT_WEAPONS] |= ( 1 << WP_GAUNTLET );
+	              	client->ps.ammo[WP_GAUNTLET] = -1;
+		}
+	}
+
+	//nexuiz style rocket arena (rocket launcher only)
+	if(g_rockets.integer) 
+	{
+		client->ps.stats[STAT_WEAPONS] = ( 1 << WP_ROCKET_LAUNCHER );
+		client->ps.ammo[WP_ROCKET_LAUNCHER] = 999;
+	}
 
 	G_SetOrigin( ent, spawn_origin );
 	VectorCopy( spawn_origin, client->ps.origin );
 
 	// the respawned flag will be cleared after the attack and jump keys come up
 	client->ps.pm_flags |= PMF_RESPAWNED;
+	if(g_gametype.integer==GT_ELIMINATION || g_gametype.integer==GT_CTF_ELIMINATION || g_gametype.integer==GT_LMS)	
+		client->ps.pm_flags |= PMF_ELIMWARMUP;
 
 	trap_GetUsercmd( client - level.clients, &ent->client->pers.cmd );
 	SetClientViewAngle( ent, spawn_angles );
 
-	if ( ent->client->sess.sessionTeam == TEAM_SPECTATOR ) {
-
+	if ( (ent->client->sess.sessionTeam == TEAM_SPECTATOR) || ((client->ps.pm_type == PM_SPECTATOR || client->isEliminated) && 
+		(g_gametype.integer == GT_ELIMINATION || g_gametype.integer == GT_CTF_ELIMINATION || g_gametype.integer == GT_LMS) ) ) {
+                //Sago: Lets see if this fixes the bots only bug - loose all point on dead bug. (It didn't)
+            /*if(g_gametype.integer == GT_ELIMINATION || g_gametype.integer == GT_CTF_ELIMINATION || g_gametype.integer == GT_LMS) {
+                G_KillBox( ent );
+		trap_LinkEntity (ent);
+            }*/
 	} else {
 		G_KillBox( ent );
 		trap_LinkEntity (ent);
@@ -1247,7 +1780,8 @@ void ClientSpawn(gentity_t *ent) {
 	ClientThink( ent-g_entities );
 
 	// positively link the client, even if the command times are weird
-	if ( ent->client->sess.sessionTeam != TEAM_SPECTATOR ) {
+	if ( (ent->client->sess.sessionTeam != TEAM_SPECTATOR) || ( (!client->isEliminated || client->ps.pm_type != PM_SPECTATOR)&& 
+		(g_gametype.integer == GT_ELIMINATION || g_gametype.integer == GT_CTF_ELIMINATION || g_gametype.integer == GT_LMS) ) ) {
 		BG_PlayerStateToEntityState( &client->ps, &ent->s, qtrue );
 		VectorCopy( ent->client->ps.origin, ent->r.currentOrigin );
 		trap_LinkEntity( ent );
@@ -1289,7 +1823,7 @@ void ClientDisconnect( int clientNum ) {
 
 	// stop any following clients
 	for ( i = 0 ; i < level.maxclients ; i++ ) {
-		if ( level.clients[i].sess.sessionTeam == TEAM_SPECTATOR
+		if ( (level.clients[i].sess.sessionTeam == TEAM_SPECTATOR || level.clients[i].ps.pm_type == PM_SPECTATOR)
 			&& level.clients[i].sess.spectatorState == SPECTATOR_FOLLOW
 			&& level.clients[i].sess.spectatorClient == clientNum ) {
 			StopFollowing( &g_entities[i] );
