@@ -53,7 +53,10 @@ void DeathmatchScoreboardMessage( gentity_t *ent ) {
 		if ( cl->pers.connected == CON_CONNECTING ) {
 			ping = -1;
 		} else {
-			ping = cl->ps.ping < 999 ? cl->ps.ping : 999;
+//unlagged - true ping
+			//ping = cl->ps.ping < 999 ? cl->ps.ping : 999;
+			ping = cl->pers.realPing < 999 ? cl->pers.realPing : 999;
+//unlagged - true ping
 		}
 
 		if( cl->accuracy_shots ) {
@@ -64,17 +67,34 @@ void DeathmatchScoreboardMessage( gentity_t *ent ) {
 		}
 		perfect = ( cl->ps.persistant[PERS_RANK] == 0 && cl->ps.persistant[PERS_KILLED] == 0 ) ? 1 : 0;
 
-		Com_sprintf (entry, sizeof(entry),
-			" %i %i %i %i %i %i %i %i %i %i %i %i %i %i", level.sortedClients[i],
-			cl->ps.persistant[PERS_SCORE], ping, (level.time - cl->pers.enterTime)/60000,
-			scoreFlags, g_entities[level.sortedClients[i]].s.powerups, accuracy, 
-			cl->ps.persistant[PERS_IMPRESSIVE_COUNT],
-			cl->ps.persistant[PERS_EXCELLENT_COUNT],
-			cl->ps.persistant[PERS_GAUNTLET_FRAG_COUNT], 
-			cl->ps.persistant[PERS_DEFEND_COUNT], 
-			cl->ps.persistant[PERS_ASSIST_COUNT], 
-			perfect,
-			cl->ps.persistant[PERS_CAPTURES]);
+		if(g_gametype.integer == GT_LMS) {
+			Com_sprintf (entry, sizeof(entry),
+				" %i %i %i %i %i %i %i %i %i %i %i %i %i %i %i", level.sortedClients[i],
+				cl->ps.persistant[PERS_SCORE], ping, (level.time - cl->pers.enterTime)/60000,
+				scoreFlags, g_entities[level.sortedClients[i]].s.powerups, accuracy, 
+				cl->ps.persistant[PERS_IMPRESSIVE_COUNT],
+				cl->ps.persistant[PERS_EXCELLENT_COUNT],
+				cl->ps.persistant[PERS_GAUNTLET_FRAG_COUNT], 
+				cl->ps.persistant[PERS_DEFEND_COUNT], 
+				cl->ps.persistant[PERS_ASSIST_COUNT], 
+				perfect,
+				cl->ps.persistant[PERS_CAPTURES],
+				cl->pers.livesLeft + (cl->isEliminated?0:1));
+		}
+		else {
+			Com_sprintf (entry, sizeof(entry),
+				" %i %i %i %i %i %i %i %i %i %i %i %i %i %i %i", level.sortedClients[i],
+				cl->ps.persistant[PERS_SCORE], ping, (level.time - cl->pers.enterTime)/60000,
+				scoreFlags, g_entities[level.sortedClients[i]].s.powerups, accuracy, 
+				cl->ps.persistant[PERS_IMPRESSIVE_COUNT],
+				cl->ps.persistant[PERS_EXCELLENT_COUNT],
+				cl->ps.persistant[PERS_GAUNTLET_FRAG_COUNT], 
+				cl->ps.persistant[PERS_DEFEND_COUNT], 
+				cl->ps.persistant[PERS_ASSIST_COUNT], 
+				perfect,
+				cl->ps.persistant[PERS_CAPTURES],
+				cl->isEliminated);
+		}
 		j = strlen(entry);
 		if (stringlength + j > 1024)
 			break;
@@ -82,11 +102,102 @@ void DeathmatchScoreboardMessage( gentity_t *ent ) {
 		stringlength += j;
 	}
 
-	trap_SendServerCommand( ent-g_entities, va("scores %i %i %i%s", i, 
-		level.teamScores[TEAM_RED], level.teamScores[TEAM_BLUE],
+	trap_SendServerCommand( ent-g_entities, va("scores %i %i %i %i%s", i, 
+		level.teamScores[TEAM_RED], level.teamScores[TEAM_BLUE], level.roundStartTime,
 		string ) );
 }
 
+/*
+==================
+DominationPointStatusMessage
+
+==================
+*/
+void DominationPointStatusMessage( gentity_t *ent ) {
+	char		entry[20]; //Will more likely be 2... in fact cannot be more since we are the server
+	char		string[20*(MAX_DOMINATION_POINTS+1)];
+	int			stringlength;
+	int i, j;
+
+	string[0] = 0;
+	stringlength = 0;
+
+	for(i = 0;i<MAX_DOMINATION_POINTS && i<level.domination_points_count; i++) {
+		Com_sprintf (entry, sizeof(entry)," %i",level.pointStatusDom[i]);
+		j = strlen(entry);
+		if (stringlength + j > 20*MAX_DOMINATION_POINTS)
+			break;
+		strcpy (string + stringlength, entry);
+		stringlength += j;
+	}
+
+	trap_SendServerCommand( ent-g_entities, va("domStatus %i%s", level.domination_points_count, string ) );
+}
+
+/*
+==================
+EliminationMessage
+
+==================
+*/
+
+void EliminationMessage(gentity_t *ent) {
+	trap_SendServerCommand( ent-g_entities, va("elimination %i %i %i", 
+		level.teamScores[TEAM_RED], level.teamScores[TEAM_BLUE], level.roundStartTime) );
+}
+
+/*
+==================
+DoubleDominationScoreTime
+
+==================
+*/
+void DoubleDominationScoreTimeMessage( gentity_t *ent ) {
+	trap_SendServerCommand( ent-g_entities, va("ddtaken %i", level.timeTaken));
+}
+
+/*
+==================
+DominationPointNames
+==================
+*/
+
+void DominationPointNamesMessage( gentity_t *ent ) {
+	char text[MAX_DOMINATION_POINTS_NAMES*MAX_DOMINATION_POINTS];
+	int i,j;	
+	qboolean nullFound;
+	for(i=0;i<MAX_DOMINATION_POINTS;i++) {
+		Q_strncpyz(text+i*MAX_DOMINATION_POINTS_NAMES,level.domination_points_names[i],MAX_DOMINATION_POINTS_NAMES-1);
+		if(i!=MAX_DOMINATION_POINTS-1) {
+			//Don't allow "/0"!
+			nullFound = qfalse;
+			for(j=i*MAX_DOMINATION_POINTS_NAMES; j<(i+1)*MAX_DOMINATION_POINTS_NAMES;j++) {
+				if(text[j]==0)
+					nullFound = qtrue;
+				if(nullFound)
+					text[j] = ' ';
+			}
+		}
+		text[MAX_DOMINATION_POINTS_NAMES*MAX_DOMINATION_POINTS-2]=0x19;
+		text[MAX_DOMINATION_POINTS_NAMES*MAX_DOMINATION_POINTS-1]=0;
+	}
+	trap_SendServerCommand( ent-g_entities, va("dompointnames %i \"%s\"", level.domination_points_count, text));
+}
+
+/*
+==================
+AttackingTeamMessage
+
+==================
+*/
+void AttackingTeamMessage( gentity_t *ent ) {
+	int team;
+	if ( (level.eliminationSides+level.roundNumber)%2 == 0 )
+		team = TEAM_RED;
+	else
+		team = TEAM_BLUE;
+	trap_SendServerCommand( ent-g_entities, va("attackingteam %i", team));
+}
 
 /*
 ==================
@@ -462,7 +573,7 @@ Cmd_Kill_f
 =================
 */
 void Cmd_Kill_f( gentity_t *ent ) {
-	if ( ent->client->sess.sessionTeam == TEAM_SPECTATOR ) {
+	if ( (ent->client->sess.sessionTeam == TEAM_SPECTATOR) || ent->client->isEliminated ) {
 		return;
 	}
 	if (ent->health <= 0) {
@@ -532,7 +643,7 @@ void SetTeam( gentity_t *ent, char *s ) {
 	} else if ( !Q_stricmp( s, "spectator" ) || !Q_stricmp( s, "s" ) ) {
 		team = TEAM_SPECTATOR;
 		specState = SPECTATOR_FREE;
-	} else if ( g_gametype.integer >= GT_TEAM ) {
+	} else if ( g_gametype.integer >= GT_TEAM && g_ffa_gt!=1) {
 		// if running a team game, assign player to one of the teams
 		specState = SPECTATOR_NOT;
 		if ( !Q_stricmp( s, "red" ) || !Q_stricmp( s, "r" ) ) {
@@ -644,8 +755,16 @@ to free floating spectator mode
 =================
 */
 void StopFollowing( gentity_t *ent ) {
-	ent->client->ps.persistant[ PERS_TEAM ] = TEAM_SPECTATOR;	
-	ent->client->sess.sessionTeam = TEAM_SPECTATOR;	
+	if(g_gametype.integer<GT_ELIMINATION || g_gametype.integer>GT_LMS)
+	{
+		//Shouldn't this already be the case?
+		ent->client->ps.persistant[ PERS_TEAM ] = TEAM_SPECTATOR;	
+		ent->client->sess.sessionTeam = TEAM_SPECTATOR;	
+	}
+	else {
+		ent->client->ps.stats[STAT_HEALTH] = 0;
+		ent->health = 0;
+	}
 	ent->client->sess.spectatorState = SPECTATOR_FREE;
 	ent->client->ps.pm_flags &= ~PMF_FOLLOW;
 	ent->r.svFlags &= ~SVF_BOT;
@@ -708,6 +827,10 @@ void Cmd_Follow_f( gentity_t *ent ) {
 	int		i;
 	char	arg[MAX_TOKEN_CHARS];
 
+	//Don't folow in elimination mode, or player will get wrong score
+	/*if(ent->client->sess.sessionTeam != TEAM_SPECTATOR && g_gametype.integer>=GT_ELIMINATION && g_gametype.integer<=GT_LMS)
+		return;*/
+	
 	if ( trap_Argc() != 2 ) {
 		if ( ent->client->sess.spectatorState == SPECTATOR_FOLLOW ) {
 			StopFollowing( ent );
@@ -715,19 +838,22 @@ void Cmd_Follow_f( gentity_t *ent ) {
 		return;
 	}
 
+
 	trap_Argv( 1, arg, sizeof( arg ) );
 	i = ClientNumberFromString( ent, arg );
 	if ( i == -1 ) {
 		return;
 	}
 
+	
+
 	// can't follow self
 	if ( &level.clients[ i ] == ent->client ) {
 		return;
 	}
 
-	// can't follow another spectator
-	if ( level.clients[ i ].sess.sessionTeam == TEAM_SPECTATOR ) {
+	// can't follow another spectator (or an eliminated player)
+	if ( (level.clients[ i ].sess.sessionTeam == TEAM_SPECTATOR) || level.clients[ i ].isEliminated) {
 		return;
 	}
 
@@ -738,7 +864,8 @@ void Cmd_Follow_f( gentity_t *ent ) {
 	}
 
 	// first set them to spectator
-	if ( ent->client->sess.sessionTeam != TEAM_SPECTATOR ) {
+	//if ( ent->client->sess.sessionTeam != TEAM_SPECTATOR ) {
+	if ( ent->client->sess.spectatorState == SPECTATOR_NOT ) {
 		SetTeam( ent, "spectator" );
 	}
 
@@ -786,7 +913,7 @@ void Cmd_FollowCycle_f( gentity_t *ent, int dir ) {
 		}
 
 		// can't follow another spectator
-		if ( level.clients[ clientnum ].sess.sessionTeam == TEAM_SPECTATOR ) {
+		if ( (level.clients[ clientnum ].sess.sessionTeam == TEAM_SPECTATOR) || level.clients[ clientnum ].isEliminated) {
 			continue;
 		}
 
@@ -845,7 +972,7 @@ void G_Say( gentity_t *ent, gentity_t *target, int mode, const char *chatText )
 	char		text[MAX_SAY_TEXT];
 	char		location[64];
 
-	if ( g_gametype.integer < GT_TEAM && mode == SAY_TEAM ) {
+	if ( (g_gametype.integer < GT_TEAM || g_ffa_gt == 1) && mode == SAY_TEAM ) {
 		mode = SAY_ALL;
 	}
 
@@ -867,7 +994,7 @@ void G_Say( gentity_t *ent, gentity_t *target, int mode, const char *chatText )
 		color = COLOR_CYAN;
 		break;
 	case SAY_TELL:
-		if (target && g_gametype.integer >= GT_TEAM &&
+		if (target && g_gametype.integer >= GT_TEAM && g_ffa_gt != 1 &&
 			target->client->sess.sessionTeam == ent->client->sess.sessionTeam &&
 			Team_GetLocationMsg(ent, location, sizeof(location)))
 			Com_sprintf (name, sizeof(name), EC"[%s%c%c"EC"] (%s)"EC": ", ent->client->pers.netname, Q_COLOR_ESCAPE, COLOR_WHITE, location );
@@ -1000,7 +1127,7 @@ void G_Voice( gentity_t *ent, gentity_t *target, int mode, const char *id, qbool
 	int			j;
 	gentity_t	*other;
 
-	if ( g_gametype.integer < GT_TEAM && mode == SAY_TEAM ) {
+	if ( (g_gametype.integer < GT_TEAM || g_ffa_gt==1 ) && mode == SAY_TEAM ) {
 		mode = SAY_ALL;
 	}
 
@@ -1133,7 +1260,7 @@ static void Cmd_VoiceTaunt_f( gentity_t *ent ) {
 		}
 	}
 
-	if (g_gametype.integer >= GT_TEAM) {
+	if (g_gametype.integer >= GT_TEAM && g_ffa_gt!=1) {
 		// praise a team mate who just got a reward
 		for(i = 0; i < MAX_CLIENTS; i++) {
 			who = g_entities + i;
@@ -1204,7 +1331,12 @@ static const char *gameNames[] = {
 	"Capture the Flag",
 	"One Flag CTF",
 	"Overload",
-	"Harvester"
+	"Harvester",
+	"Elimination",
+	"CTF Elimination",
+	"Last Man Standing",
+	"Double Domination",
+	"Domination"
 };
 
 /*
@@ -1696,6 +1828,8 @@ void ClientCommand( int clientNum ) {
 		Cmd_SetViewpos_f( ent );
 	else if (Q_stricmp (cmd, "stats") == 0)
 		Cmd_Stats_f( ent );
+	else if (Q_stricmp (cmd, "free_spectator") == 0)
+		StopFollowing( ent );
 	else
 		trap_SendServerCommand( clientNum, va("print \"unknown cmd %s\n\"", cmd ) );
 }
