@@ -94,6 +94,57 @@ vmCvar_t	g_enableDust;
 vmCvar_t	g_enableBreath;
 vmCvar_t	g_proxMineTimeout;
 #endif
+//Following for elimination:
+vmCvar_t	g_elimination_selfdamage;
+vmCvar_t	g_elimination_startHealth;
+vmCvar_t	g_elimination_startArmor;
+vmCvar_t	g_elimination_bfg;
+vmCvar_t	g_elimination_roundtime;
+vmCvar_t	g_elimination_warmup;
+vmCvar_t	g_elimination_activewarmup;
+//more weapons in beta 9:
+vmCvar_t	g_elimination_machinegun;
+vmCvar_t	g_elimination_shotgun;
+vmCvar_t	g_elimination_grenade;
+vmCvar_t	g_elimination_rocket;
+vmCvar_t	g_elimination_railgun;
+vmCvar_t	g_elimination_lightning;
+vmCvar_t	g_elimination_plasmagun;
+#ifdef MISSIONPACK
+vmCvar_t	g_elimination_chain;
+vmCvar_t	g_elimination_mine;
+vmCvar_t	g_elimination_nail;
+#endif
+
+vmCvar_t	g_rockets;
+
+//dmn_clowns suggestions (with my idea of implementing):
+vmCvar_t	g_instantgib;
+vmCvar_t	g_vampire;
+vmCvar_t	g_vampireMaxHealth;
+//Regen
+vmCvar_t	g_regen;
+//Beta 4
+int	g_ffa_gt; //Are this a FFA gametype even if gametype is high?
+//beta 5
+vmCvar_t	g_lms_lives;
+
+//beta 8
+vmCvar_t	g_lms_mode;
+
+//beta 10
+vmCvar_t	g_elimination_ctf_oneway;
+
+//beta 18 (oa 0.7.6)
+vmCvar_t        g_awardpushing; //The server can decide if players are awarded for pushing people in lave etc.
+
+//unlagged - server options
+vmCvar_t	g_delagHitscan;
+vmCvar_t	g_unlaggedVersion;
+vmCvar_t	g_truePing;
+vmCvar_t	g_lightningDamage;
+vmCvar_t	sv_fps;
+//unlagged - server options
 
 // bk001129 - made static to avoid aliasing
 static cvarTable_t		gameCvarTable[] = {
@@ -127,8 +178,8 @@ static cvarTable_t		gameCvarTable[] = {
 
 	{ &g_warmup, "g_warmup", "20", CVAR_ARCHIVE, 0, qtrue  },
 	{ &g_doWarmup, "g_doWarmup", "0", 0, 0, qtrue  },
-	{ &g_logfile, "g_logfile", "games.log", CVAR_ARCHIVE, 0, qfalse  },
-	{ &g_logfileSync, "g_logfileSync", "0", CVAR_ARCHIVE, 0, qfalse  },
+	{ &g_logfile, "g_log", "games.log", CVAR_ARCHIVE, 0, qfalse  },
+	{ &g_logfileSync, "g_logsync", "0", CVAR_ARCHIVE, 0, qfalse  },
 
 	{ &g_password, "g_password", "", CVAR_USERINFO, 0, qfalse  },
 
@@ -178,8 +229,55 @@ static cvarTable_t		gameCvarTable[] = {
 	{ &pmove_fixed, "pmove_fixed", "0", CVAR_SYSTEMINFO, 0, qfalse},
 	{ &pmove_msec, "pmove_msec", "8", CVAR_SYSTEMINFO, 0, qfalse},
 
-	{ &g_rankings, "g_rankings", "0", 0, 0, qfalse}
+//unlagged - server options
+	{ &g_delagHitscan, "g_delagHitscan", "0", CVAR_ARCHIVE | CVAR_SERVERINFO, 0, qtrue },
+	{ &g_unlaggedVersion, "g_unlaggedVersion", "2.0", CVAR_ROM | CVAR_SERVERINFO, 0, qfalse },
+	{ &g_truePing, "g_truePing", "0", CVAR_ARCHIVE, 0, qtrue },
+	{ &g_lightningDamage, "g_lightningDamage", "8", 0, 0, qtrue },
+	// it's CVAR_SYSTEMINFO so the client's sv_fps will be automagically set to its value
+	{ &sv_fps, "sv_fps", "20", CVAR_SYSTEMINFO | CVAR_ARCHIVE, 0, qfalse },
+//unlagged - server options
 
+	{ &g_rankings, "g_rankings", "0", 0, 0, qfalse},
+	//Now for elimination stuff:
+	{ &g_elimination_selfdamage, "elimination_selfdamage", "0", 0, 0, qtrue },
+	{ &g_elimination_startHealth, "elimination_startHealth", "200", CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_startArmor, "elimination_startArmor", "200", CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_bfg, "elimination_bfg", "0", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_roundtime, "elimination_roundtime", "120", CVAR_SERVERINFO | CVAR_ARCHIVE | CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_warmup, "elimination_warmup", "7", 0, 0, qtrue },
+	{ &g_elimination_activewarmup, "elimination_activewarmup", "5", 0, 0, qtrue },
+//more weapons in beta 9:
+	{ &g_elimination_machinegun, "elimination_machinegun", "200", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_shotgun, "elimination_shotgun", "200", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_grenade, "elimination_grenade", "200", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_rocket, "elimination_rocket", "200", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_railgun, "elimination_railgun", "200", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_lightning, "elimination_lightning", "200", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_plasmagun, "elimination_plasmagun", "200", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+#ifdef MISSIONPACK
+	{ &g_elimination_chain, "elimination_chain", "0", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_mine, "elimination_mine", "0", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+	{ &g_elimination_nail, "elimination_nail", "0", CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+#endif
+	//Beta 10
+	{ &g_elimination_ctf_oneway, "elimination_ctf_oneway", "0", CVAR_SERVERINFO | CVAR_ARCHIVE| CVAR_NORESTART, 0, qtrue },
+
+        
+        { &g_awardpushing, "g_awardpushing", "0", CVAR_ARCHIVE | CVAR_NORESTART, 0, qtrue },    
+        
+	//nexuiz style rocket arena
+	{ &g_rockets, "g_rockets", "0", CVAR_SERVERINFO | CVAR_LATCH | CVAR_NORESTART, 0, qfalse },
+
+	//Instantgib and Vampire thingies
+	{ &g_instantgib, "g_instantgib", "0", CVAR_SERVERINFO | CVAR_LATCH | CVAR_NORESTART, 0, qfalse },
+	{ &g_vampire, "g_vampire", "0.0", CVAR_NORESTART, 0, qfalse },
+	{ &g_regen, "g_regen", "0", CVAR_NORESTART, 0, qfalse },
+	{ &g_vampireMaxHealth, "g_vampire_max_health", "500", CVAR_NORESTART, 0, qtrue },
+	//beta 5
+	{ &g_lms_lives, "g_lms_lives", "1", CVAR_NORESTART, 0, qtrue },
+	//beta 8
+	{ &g_lms_mode, "g_lms_mode", "0", CVAR_ARCHIVE | CVAR_SERVERINFO | CVAR_NORESTART, 0, qtrue },
 };
 
 // bk001129 - made static to avoid aliasing
@@ -243,7 +341,7 @@ void QDECL G_Printf( const char *fmt, ... ) {
 	char		text[1024];
 
 	va_start (argptr, fmt);
-	vsprintf (text, fmt, argptr);
+	Q_vsnprintf (text, sizeof(text), fmt, argptr);
 	va_end (argptr);
 
 	trap_Printf( text );
@@ -254,7 +352,7 @@ void QDECL G_Error( const char *fmt, ... ) {
 	char		text[1024];
 
 	va_start (argptr, fmt);
-	vsprintf (text, fmt, argptr);
+	Q_vsnprintf (text, sizeof(text), fmt, argptr);
 	va_end (argptr);
 
 	trap_Error( text );
@@ -362,6 +460,13 @@ void G_RegisterCvars( void ) {
 		trap_Cvar_Set( "g_gametype", "0" );
 	}
 
+	//set FFA status for high gametypes:
+	if ( g_gametype.integer == GT_LMS ) {
+		g_ffa_gt = 1;	//Last Man standig is a FFA gametype
+	} else {
+		g_ffa_gt = 0;	//If >GT_CTF use bases
+	}
+
 	level.warmupModificationCount = g_warmup.modificationCount;
 }
 
@@ -484,7 +589,7 @@ void G_InitGame( int levelTime, int randomSeed, int restart ) {
 	G_FindTeams();
 
 	// make sure we have flags for CTF, etc
-	if( g_gametype.integer >= GT_TEAM ) {
+	if( g_gametype.integer >= GT_TEAM && (g_ffa_gt!=1)) {
 		G_CheckTeamItems();
 	}
 
@@ -506,6 +611,22 @@ void G_InitGame( int levelTime, int randomSeed, int restart ) {
 
 	G_RemapTeamShaders();
 
+	//elimination:
+	level.roundNumber = 1;
+	level.roundNumberStarted = 0;
+	level.roundStartTime = level.time+g_elimination_warmup.integer*1000;
+	level.roundRespawned = qfalse;
+	level.eliminationSides = rand()%2; //0 or 1
+
+	if(g_gametype.integer == GT_DOUBLE_D)
+		Team_SpawnDoubleDominationPoints();
+
+	if(g_gametype.integer == GT_DOMINATION ){
+		level.dom_scoreGiven = 0;
+		for(i=0;i<MAX_DOMINATION_POINTS;i++)
+			level.pointStatusDom[i] = TEAM_NONE;
+		level.domination_points_count = 0; //make sure its not too big
+	}
 }
 
 
@@ -541,7 +662,7 @@ void QDECL Com_Error ( int level, const char *error, ... ) {
 	char		text[1024];
 
 	va_start (argptr, error);
-	vsprintf (text, error, argptr);
+	Q_vsnprintf (text, sizeof(text), error, argptr);
 	va_end (argptr);
 
 	G_Error( "%s", text);
@@ -552,7 +673,7 @@ void QDECL Com_Printf( const char *msg, ... ) {
 	char		text[1024];
 
 	va_start (argptr, msg);
-	vsprintf (text, msg, argptr);
+	Q_vsnprintf (text, sizeof(text), msg, argptr);
 	va_end (argptr);
 
 	G_Printf ("%s", text);
@@ -802,7 +923,7 @@ void CalculateRanks( void ) {
 		sizeof(level.sortedClients[0]), SortRanks );
 
 	// set the rank value for all clients that are connected and not spectators
-	if ( g_gametype.integer >= GT_TEAM ) {
+	if ( g_gametype.integer >= GT_TEAM && g_ffa_gt!=1) {
 		// in team games, rank is just the order of the teams, 0=red, 1=blue, 2=tied
 		for ( i = 0;  i < level.numConnectedClients; i++ ) {
 			cl = &level.clients[ level.sortedClients[i] ];
@@ -837,7 +958,7 @@ void CalculateRanks( void ) {
 	}
 
 	// set the CS_SCORES1/2 configstrings, which will be visible to everyone
-	if ( g_gametype.integer >= GT_TEAM ) {
+	if ( g_gametype.integer >= GT_TEAM && g_ffa_gt!=1) {
 		trap_SetConfigstring( CS_SCORES1, va("%i", level.teamScores[TEAM_RED] ) );
 		trap_SetConfigstring( CS_SCORES2, va("%i", level.teamScores[TEAM_BLUE] ) );
 	} else {
@@ -885,6 +1006,75 @@ void SendScoreboardMessageToAllClients( void ) {
 	for ( i = 0 ; i < level.maxclients ; i++ ) {
 		if ( level.clients[ i ].pers.connected == CON_CONNECTED ) {
 			DeathmatchScoreboardMessage( g_entities + i );
+			EliminationMessage( g_entities + i );
+		}
+	}
+}
+
+/*
+========================
+SendElimiantionMessageToAllClients
+
+Used to send information important to Elimination
+========================
+*/
+void SendEliminationMessageToAllClients( void ) {
+	int		i;
+
+	for ( i = 0 ; i < level.maxclients ; i++ ) {
+		if ( level.clients[ i ].pers.connected == CON_CONNECTED ) {
+			EliminationMessage( g_entities + i );
+		}
+	}
+}
+
+/*
+========================
+SendDDtimetakenMessageToAllClients
+
+Do this if a team just started dominating.
+========================
+*/
+void SendDDtimetakenMessageToAllClients( void ) {
+	int		i;
+
+	for ( i = 0 ; i < level.maxclients ; i++ ) {
+		if ( level.clients[ i ].pers.connected == CON_CONNECTED ) {
+			DoubleDominationScoreTimeMessage( g_entities + i );
+		}
+	}
+}
+
+/*
+========================
+SendAttackingTeamMessageToAllClients
+
+Used for CTF Elimination oneway
+========================
+*/
+void SendAttackingTeamMessageToAllClients( void ) {
+	int		i;
+
+	for ( i = 0 ; i < level.maxclients ; i++ ) {
+		if ( level.clients[ i ].pers.connected == CON_CONNECTED ) {
+			AttackingTeamMessage( g_entities + i );
+		}
+	}
+}
+
+/*
+========================
+SendDominationPointsStatusMessageToAllClients
+
+Used for Standard domination
+========================
+*/
+void SendDominationPointsStatusMessageToAllClients( void ) {
+	int		i;
+
+	for ( i = 0 ; i < level.maxclients ; i++ ) {
+		if ( level.clients[ i ].pers.connected == CON_CONNECTED ) {
+			DominationPointStatusMessage( g_entities + i );
 		}
 	}
 }
@@ -1094,7 +1284,7 @@ void QDECL G_LogPrintf( const char *fmt, ... ) {
 	Com_sprintf( string, sizeof(string), "%3i:%i%i ", min, tens, sec );
 
 	va_start( argptr, fmt );
-	vsprintf( string +7 , fmt,argptr );
+	Q_vsnprintf(string + 7, sizeof(string - 7), fmt, argptr);
 	va_end( argptr );
 
 	if ( g_dedicated.integer ) {
@@ -1135,7 +1325,7 @@ void LogExit( const char *string ) {
 		numSorted = 32;
 	}
 
-	if ( g_gametype.integer >= GT_TEAM ) {
+	if ( g_gametype.integer >= GT_TEAM && g_ffa_gt!=1) {
 		G_LogPrintf( "red:%i  blue:%i\n",
 			level.teamScores[TEAM_RED], level.teamScores[TEAM_BLUE] );
 	}
@@ -1167,7 +1357,7 @@ void LogExit( const char *string ) {
 
 #ifdef MISSIONPACK
 	if (g_singlePlayer.integer) {
-		if (g_gametype.integer >= GT_CTF) {
+		if (g_gametype.integer >= GT_CTF && g_ffa_gt==0) {
 			won = level.teamScores[TEAM_RED] > level.teamScores[TEAM_BLUE];
 		}
 		trap_SendConsoleCommand( EXEC_APPEND, (won) ? "spWin\n" : "spLose\n" );
@@ -1189,7 +1379,7 @@ wait 10 seconds before going on.
 =================
 */
 void CheckIntermissionExit( void ) {
-	int			ready, notReady;
+	int			ready, notReady, playerCount;
 	int			i;
 	gclient_t	*cl;
 	int			readyMask;
@@ -1202,6 +1392,7 @@ void CheckIntermissionExit( void ) {
 	ready = 0;
 	notReady = 0;
 	readyMask = 0;
+        playerCount = 0;
 	for (i=0 ; i< g_maxclients.integer ; i++) {
 		cl = level.clients + i;
 		if ( cl->pers.connected != CON_CONNECTED ) {
@@ -1211,6 +1402,7 @@ void CheckIntermissionExit( void ) {
 			continue;
 		}
 
+                playerCount++;
 		if ( cl->readyToExit ) {
 			ready++;
 			if ( i < 16 ) {
@@ -1236,16 +1428,19 @@ void CheckIntermissionExit( void ) {
 		return;
 	}
 
-	// if nobody wants to go, clear timer
-	if ( !ready ) {
-		level.readyToExit = qfalse;
-		return;
-	}
+	// only test ready status when there are real players present
+	if ( playerCount > 0 ) {
+		// if nobody wants to go, clear timer
+		if ( !ready ) {
+			level.readyToExit = qfalse;
+			return;
+		}
 
-	// if everyone wants to go, go now
-	if ( !notReady ) {
-		ExitLevel();
-		return;
+		// if everyone wants to go, go now
+		if ( !notReady ) {
+			ExitLevel();
+			return;
+		}
 	}
 
 	// the first person to ready starts the ten second timeout
@@ -1275,7 +1470,7 @@ qboolean ScoreIsTied( void ) {
 		return qfalse;
 	}
 	
-	if ( g_gametype.integer >= GT_TEAM ) {
+	if ( g_gametype.integer >= GT_TEAM && g_ffa_gt!=1) {
 		return level.teamScores[TEAM_RED] == level.teamScores[TEAM_BLUE];
 	}
 
@@ -1338,7 +1533,7 @@ void CheckExitRules( void ) {
 		return;
 	}
 
-	if ( g_gametype.integer < GT_CTF && g_fraglimit.integer ) {
+	if ( (g_gametype.integer < GT_CTF || g_ffa_gt>0 ) && g_fraglimit.integer ) {
 		if ( level.teamScores[TEAM_RED] >= g_fraglimit.integer ) {
 			trap_SendServerCommand( -1, "print \"Red hit the fraglimit.\n\"" );
 			LogExit( "Fraglimit hit." );
@@ -1369,7 +1564,7 @@ void CheckExitRules( void ) {
 		}
 	}
 
-	if ( g_gametype.integer >= GT_CTF && g_capturelimit.integer ) {
+	if ( (g_gametype.integer >= GT_CTF && g_ffa_gt<1) && g_capturelimit.integer ) {
 
 		if ( level.teamScores[TEAM_RED] >= g_capturelimit.integer ) {
 			trap_SendServerCommand( -1, "print \"Red hit the capturelimit.\n\"" );
@@ -1385,7 +1580,89 @@ void CheckExitRules( void ) {
 	}
 }
 
+//LMS - Last man Stading functions:
+void StartLMSRound(void) {
+	int		countsLiving;
+	countsLiving = TeamLivingCount( -1, TEAM_FREE );
+	if(countsLiving<2) {
+		trap_SendServerCommand( -1, "print \"Not enough players to start the round\n\"");
+		level.roundNumberStarted = level.roundNumber-1;
+		level.roundStartTime = level.time+1000*g_elimination_warmup.integer;
+		return;
+	}
 
+	//If we are enough to start a round:
+	level.roundNumberStarted = level.roundNumber; //Set numbers
+
+	SendEliminationMessageToAllClients();
+	EnableWeapons();
+}
+
+//the elimination start function
+void StartEliminationRound(void) {
+
+	int		countsLiving[TEAM_NUM_TEAMS];
+	countsLiving[TEAM_BLUE] = TeamLivingCount( -1, TEAM_BLUE );
+	countsLiving[TEAM_RED] = TeamLivingCount( -1, TEAM_RED );
+	if((countsLiving[TEAM_BLUE]==0) || (countsLiving[TEAM_RED]==0))
+	{
+		trap_SendServerCommand( -1, "print \"Not enough players to start the round\n\"");
+		level.roundNumberStarted = level.roundNumber-1;
+		level.roundRespawned = qfalse;
+		//Remember that one of the teams is empty!
+		level.roundRedPlayers = countsLiving[TEAM_RED];
+		level.roundBluePlayers = countsLiving[TEAM_BLUE];
+		level.roundStartTime = level.time+1000*g_elimination_warmup.integer;
+		return;
+	}
+	
+	//If we are enough to start a round:
+	level.roundNumberStarted = level.roundNumber; //Set numbers
+	level.roundRedPlayers = countsLiving[TEAM_RED];
+	level.roundBluePlayers = countsLiving[TEAM_BLUE];
+	if(g_gametype.integer == GT_CTF_ELIMINATION) {
+		Team_ReturnFlag( TEAM_RED );
+		Team_ReturnFlag( TEAM_BLUE );
+	}
+	SendEliminationMessageToAllClients();
+	if(g_elimination_ctf_oneway.integer)
+		SendAttackingTeamMessageToAllClients(); //Ensure that evaryone know who should attack.
+	EnableWeapons();
+}
+
+//things to do at end of round:
+void EndEliminationRound(void)
+{
+	DisableWeapons();
+	level.roundNumber++;
+	level.roundStartTime = level.time+1000*g_elimination_warmup.integer;
+	SendEliminationMessageToAllClients();
+	level.roundRespawned = qfalse;
+	if(g_elimination_ctf_oneway.integer)
+		SendAttackingTeamMessageToAllClients();
+}
+
+//Things to do if we don't want to move the roundNumber
+void RestartEliminationRound(void) {
+	DisableWeapons();
+	level.roundNumberStarted = level.roundNumber-1;
+	level.roundStartTime = level.time+1000*g_elimination_warmup.integer;
+	SendEliminationMessageToAllClients();
+	level.roundRespawned = qfalse;
+	if(g_elimination_ctf_oneway.integer)
+		SendAttackingTeamMessageToAllClients();
+}
+
+//Things to do during match warmup
+void WarmupEliminationRound(void) {
+	EnableWeapons();
+	level.roundNumberStarted = level.roundNumber-1;
+	level.roundStartTime = level.time+1000*g_elimination_warmup.integer;
+	SendEliminationMessageToAllClients();
+	level.roundRespawned = qfalse;
+	if(g_elimination_ctf_oneway.integer)
+		SendAttackingTeamMessageToAllClients();
+}
 
 /*
 ========================================================================
@@ -1395,6 +1672,330 @@ FUNCTIONS CALLED EVERY FRAME
 ========================================================================
 */
 
+/*
+CheckDoubleDomination
+*/
+
+void CheckDoubleDomination( void ) {
+	if ( level.numPlayingClients < 1 ) {
+		return;
+	}
+
+	if ( level.warmupTime != 0)
+		return;
+
+	if(g_gametype.integer != GT_DOUBLE_D)
+		return;
+
+	//Don't score if we are in intermission. Both points might have been taken when we went into intermission
+	if(level.intermissiontime)
+		return;
+
+	if(level.pointStatusA == TEAM_RED && level.pointStatusB == TEAM_RED && level.timeTaken + 10*1000 <= level.time) {
+		//Red scores
+		trap_SendServerCommand( -1, "print \"Red team scores!\n\"");
+		AddTeamScore(level.intermission_origin,TEAM_RED,1);
+		Team_ForceGesture(TEAM_RED);
+		Team_DD_bonusAtPoints(TEAM_RED);
+		Team_RemoveDoubleDominationPoints();
+		//We start again in 10 seconds:
+		level.roundStartTime = level.time + 10*1000;
+		SendScoreboardMessageToAllClients();
+		CalculateRanks();
+	}
+
+	if(level.pointStatusA == TEAM_BLUE && level.pointStatusB == TEAM_BLUE && level.timeTaken + 10*1000 <= level.time) {
+		//Blue scores
+		trap_SendServerCommand( -1, "print \"Blue team scores!\n\"");
+		AddTeamScore(level.intermission_origin,TEAM_BLUE,1);
+		Team_ForceGesture(TEAM_BLUE);
+		Team_DD_bonusAtPoints(TEAM_BLUE);
+		Team_RemoveDoubleDominationPoints();
+		//We start again in 10 seconds:
+		level.roundStartTime = level.time + 10*1000;
+		SendScoreboardMessageToAllClients();
+		CalculateRanks();
+	}
+
+	if((level.pointStatusA == TEAM_NONE || level.pointStatusB == TEAM_NONE) && level.time>level.roundStartTime) {
+		trap_SendServerCommand( -1, "print \"A new round has started\n\"");
+		Team_SpawnDoubleDominationPoints();
+		SendScoreboardMessageToAllClients();
+	}
+}
+
+/*
+CheckLMS
+*/
+
+void CheckLMS(void) {
+	int mode;
+	mode = g_lms_mode.integer;
+	if ( level.numPlayingClients < 1 ) {
+		return;
+	}
+
+	
+
+	//We don't want to do anything in intermission
+	if(level.intermissiontime) {
+		if(level.roundRespawned) {
+			RestartEliminationRound();
+		}
+		level.roundStartTime = level.time; //so that a player might join at any time to fix the bots+no humans+autojoin bug
+		return;
+	}
+
+	if(g_gametype.integer == GT_LMS)
+	{
+		int		countsLiving[TEAM_NUM_TEAMS];
+		//trap_SendServerCommand( -1, "print \"This is LMS!\n\"");
+		countsLiving[TEAM_FREE] = TeamLivingCount( -1, TEAM_FREE );
+		if(countsLiving[TEAM_FREE]==1 && level.roundNumber==level.roundNumberStarted)
+		{
+			if(mode <=1 )
+			LMSpoint();
+			trap_SendServerCommand( -1, "print \"We have a winner!\n\"");
+			EndEliminationRound();
+			Team_ForceGesture(TEAM_FREE);
+		}
+
+		if(countsLiving[TEAM_FREE]==0 && level.roundNumber==level.roundNumberStarted)
+		{
+			trap_SendServerCommand( -1, "print \"All death... how sad\n\"");
+			EndEliminationRound();
+		}
+
+		if((level.roundNumber==level.roundNumberStarted)&&(g_lms_mode.integer == 1 || g_lms_mode.integer==3)&&(level.time>=level.roundStartTime+1000*g_elimination_roundtime.integer))
+		{
+			trap_SendServerCommand( -1, "print \"Time up - Overtime disabled\n\"");
+			if(mode <=1 )
+			LMSpoint();
+			EndEliminationRound();
+		}
+
+		//This might be better placed another place:
+		if(g_elimination_activewarmup.integer<2)
+			g_elimination_activewarmup.integer=2; //We need at least 2 seconds to spawn all players
+		if(g_elimination_activewarmup.integer >= g_elimination_warmup.integer) //This must not be true
+			g_elimination_warmup.integer = g_elimination_activewarmup.integer+1; //Increase warmup
+
+		//Force respawn
+		if(level.roundNumber != level.roundNumberStarted && level.time>level.roundStartTime-1000*g_elimination_activewarmup.integer && !level.roundRespawned)
+		{
+			level.roundRespawned = qtrue;
+			RespawnAll();
+			DisableWeapons();
+			SendEliminationMessageToAllClients();
+		}
+
+		if(level.time<=level.roundStartTime && level.time>level.roundStartTime-1000*g_elimination_activewarmup.integer)
+		{
+			RespawnDead();
+			//DisableWeapons();
+		}
+
+		if(level.roundNumber == level.roundNumberStarted)
+		{
+			EnableWeapons();
+		}
+
+		if((level.roundNumber>level.roundNumberStarted)&&(level.time>=level.roundStartTime))
+			StartLMSRound();
+	
+		if(level.time+1000*g_elimination_warmup.integer-500>level.roundStartTime && level.numPlayingClients < 2)
+		{
+			RespawnDead(); //Allow player to run around anyway
+			WarmupEliminationRound(); //Start over
+			return;
+		}
+
+		if(level.warmupTime != 0) {
+			if(level.time+1000*g_elimination_warmup.integer-500>level.roundStartTime)
+			{
+				RespawnDead();
+				WarmupEliminationRound();
+			}
+		}
+
+	}
+}
+
+/*
+=============
+CheckElimination
+=============
+*/
+void CheckElimination(void) {
+	if ( level.numPlayingClients < 1 ) {
+		if( (g_gametype.integer == GT_ELIMINATION || g_gametype.integer == GT_CTF_ELIMINATION) &&
+			( level.time+1000*g_elimination_warmup.integer-500>level.roundStartTime ))
+			RestartEliminationRound(); //For spectators
+		return;
+	}	
+
+	//We don't want to do anything in itnermission
+	if(level.intermissiontime) {
+		if(level.roundRespawned)
+			RestartEliminationRound();
+		level.roundStartTime = level.time+1000*g_elimination_warmup.integer;
+		return;
+	}	
+
+	if(g_gametype.integer == GT_ELIMINATION || g_gametype.integer == GT_CTF_ELIMINATION)
+	{
+		int		counts[TEAM_NUM_TEAMS];
+		int		countsLiving[TEAM_NUM_TEAMS];
+		int		countsHealth[TEAM_NUM_TEAMS];
+		counts[TEAM_BLUE] = TeamCount( -1, TEAM_BLUE );
+		counts[TEAM_RED] = TeamCount( -1, TEAM_RED );
+
+		countsLiving[TEAM_BLUE] = TeamLivingCount( -1, TEAM_BLUE );
+		countsLiving[TEAM_RED] = TeamLivingCount( -1, TEAM_RED );
+
+		countsHealth[TEAM_BLUE] = TeamHealthCount( -1, TEAM_BLUE );
+		countsHealth[TEAM_RED] = TeamHealthCount( -1, TEAM_RED );
+
+		if(level.roundBluePlayers != 0 && level.roundRedPlayers != 0) { //Cannot score if one of the team never got any living players
+			if((countsLiving[TEAM_BLUE]==0)&&(level.roundNumber==level.roundNumberStarted))
+			{
+				//Blue team has been eliminated!
+				trap_SendServerCommand( -1, "print \"Blue Team eliminated!\n\"");
+				AddTeamScore(level.intermission_origin,TEAM_RED,1);
+				EndEliminationRound();
+				Team_ForceGesture(TEAM_RED);
+			}
+			else if((countsLiving[TEAM_RED]==0)&&(level.roundNumber==level.roundNumberStarted))
+			{
+				//Red team eliminated!
+				trap_SendServerCommand( -1, "print \"Red Team eliminated!\n\"");
+				AddTeamScore(level.intermission_origin,TEAM_BLUE,1);
+				EndEliminationRound();
+				Team_ForceGesture(TEAM_BLUE);
+			}
+		}
+
+		//Time up
+		if((level.roundNumber==level.roundNumberStarted)&&(level.time>=level.roundStartTime+1000*g_elimination_roundtime.integer))
+		{
+			trap_SendServerCommand( -1, "print \"No teams eliminated!\n\"");
+
+			if(level.roundBluePlayers != 0 && level.roundRedPlayers != 0) {//We don't want to divide by zero. (should not be possible)
+				if(g_gametype.integer == GT_CTF_ELIMINATION && g_elimination_ctf_oneway.integer) {
+					//One way CTF, make defensice team the winner.
+					if ( (level.eliminationSides+level.roundNumber)%2 == 0 ) { //Red was attacking
+						trap_SendServerCommand( -1, "print \"Blue team defended the base\n\"");
+						AddTeamScore(level.intermission_origin,TEAM_BLUE,1);
+					}
+					else {
+						trap_SendServerCommand( -1, "print \"Red team defended the base\n\"");
+						AddTeamScore(level.intermission_origin,TEAM_RED,1);
+					}
+				}
+				else if(((double)countsLiving[TEAM_RED])/((double)level.roundRedPlayers)>((double)countsLiving[TEAM_BLUE])/((double)level.roundBluePlayers))
+				{
+					//Red team has higher procentage survivors
+					trap_SendServerCommand( -1, "print \"Red team has most survivers!\n\"");
+					AddTeamScore(level.intermission_origin,TEAM_RED,1);
+				}
+				else if(((double)countsLiving[TEAM_RED])/((double)level.roundRedPlayers)<((double)countsLiving[TEAM_BLUE])/((double)level.roundBluePlayers))
+				{
+					//Blue team has higher procentage survivors
+					trap_SendServerCommand( -1, "print \"Blue team has most survivers!\n\"");
+					AddTeamScore(level.intermission_origin,TEAM_BLUE,1);	
+				}
+				else if(countsHealth[TEAM_RED]>countsHealth[TEAM_BLUE])
+				{
+					//Red team has more health
+					trap_SendServerCommand( -1, "print \"Red team has more health left!\n\"");
+					AddTeamScore(level.intermission_origin,TEAM_RED,1);
+				}
+				else if(countsHealth[TEAM_RED]<countsHealth[TEAM_BLUE])
+				{
+					//Blue team has more health
+					trap_SendServerCommand( -1, "print \"Blue team has more health left!\n\"");
+					AddTeamScore(level.intermission_origin,TEAM_BLUE,1);
+				}
+			}
+			EndEliminationRound();
+		}
+
+		//This might be better placed another place:
+		if(g_elimination_activewarmup.integer<1)
+			g_elimination_activewarmup.integer=1; //We need at least 1 second to spawn all players
+		if(g_elimination_activewarmup.integer >= g_elimination_warmup.integer) //This must not be true
+			g_elimination_warmup.integer = g_elimination_activewarmup.integer+1; //Increase warmup
+
+		//Force respawn
+		if(level.roundNumber!=level.roundNumberStarted && level.time>level.roundStartTime-1000*g_elimination_activewarmup.integer && !level.roundRespawned)
+		{
+			level.roundRespawned = qtrue;
+			RespawnAll();
+			SendEliminationMessageToAllClients();
+		}
+
+		if(level.time<=level.roundStartTime && level.time>level.roundStartTime-1000*g_elimination_activewarmup.integer)
+		{
+			RespawnDead();
+		}
+			
+
+		if((level.roundNumber>level.roundNumberStarted)&&(level.time>=level.roundStartTime))
+			StartEliminationRound();
+	
+		if(level.time+1000*g_elimination_warmup.integer-500>level.roundStartTime)
+		if(counts[TEAM_BLUE]<1 || counts[TEAM_RED]<1)
+		{
+			RespawnDead(); //Allow players to run around anyway
+			WarmupEliminationRound(); //Start over
+			return;
+		}
+
+		if(level.warmupTime != 0) {
+			if(level.time+1000*g_elimination_warmup.integer-500>level.roundStartTime)
+			{
+				RespawnDead();
+				WarmupEliminationRound();
+			}
+		}
+	}
+}
+
+/*
+=============
+CheckDomination
+=============
+*/
+void CheckDomination(void) {
+	int i;
+
+	if ( (level.numPlayingClients < 1) || (g_gametype.integer != GT_DOMINATION) ) {
+		return;
+	}
+
+	//Do nothing if warmup
+	if(level.warmupTime != 0)
+		return; 
+
+	//Don't score if we are in intermission. Just plain stupid
+	if(level.intermissiontime)
+		return;
+
+	//Sago: I use if instead of while, since if the server stops for more than 2 seconds people should not be allowed to score anyway
+	if(level.time>=level.dom_scoreGiven*DOM_SECSPERPOINT) {
+		for(i=0;i<level.domination_points_count;i++) {
+			if ( level.pointStatusDom[i] == TEAM_RED )
+				AddTeamScore(level.intermission_origin,TEAM_RED,1);
+			if ( level.pointStatusDom[i] == TEAM_BLUE )
+				AddTeamScore(level.intermission_origin,TEAM_BLUE,1);
+		}
+		level.dom_scoreGiven++;
+		while(level.time>level.dom_scoreGiven*DOM_SECSPERPOINT)
+			level.dom_scoreGiven++;
+		CalculateRanks();
+		
+	}
+}
 
 /*
 =============
@@ -1464,7 +2065,7 @@ void CheckTournament( void ) {
 		int		counts[TEAM_NUM_TEAMS];
 		qboolean	notEnough = qfalse;
 
-		if ( g_gametype.integer > GT_TEAM ) {
+		if ( g_gametype.integer > GT_TEAM && !g_ffa_gt ) {
 			counts[TEAM_BLUE] = TeamCount( -1, TEAM_BLUE );
 			counts[TEAM_RED] = TeamCount( -1, TEAM_RED );
 
@@ -1780,10 +2381,15 @@ int start, end;
 			continue;
 		}
 
+//unlagged - backward reconciliation #2
+		// we'll run missiles separately to save CPU in backward reconciliation
+/*
 		if ( ent->s.eType == ET_MISSILE ) {
 			G_RunMissile( ent );
 			continue;
 		}
+*/
+//unlagged - backward reconciliation #2
 
 		if ( ent->s.eType == ET_ITEM || ent->physicsObject ) {
 			G_RunItem( ent );
@@ -1802,6 +2408,32 @@ int start, end;
 
 		G_RunThink( ent );
 	}
+
+//unlagged - backward reconciliation #2
+	// NOW run the missiles, with all players backward-reconciled
+	// to the positions they were in exactly 50ms ago, at the end
+	// of the last server frame
+	G_TimeShiftAllClients( level.previousTime, NULL );
+
+	ent = &g_entities[0];
+	for (i=0 ; i<level.num_entities ; i++, ent++) {
+		if ( !ent->inuse ) {
+			continue;
+		}
+
+		// temporary entities don't think
+		if ( ent->freeAfterEvent ) {
+			continue;
+		}
+
+		if ( ent->s.eType == ET_MISSILE ) {
+			G_RunMissile( ent );
+		}
+	}
+
+	G_UnTimeShiftAllClients( NULL );
+//unlagged - backward reconciliation #2
+
 end = trap_Milliseconds();
 
 start = trap_Milliseconds();
@@ -1817,6 +2449,19 @@ end = trap_Milliseconds();
 	// see if it is time to do a tournement restart
 	CheckTournament();
 
+	//Check Elimination state
+	CheckElimination();
+	CheckLMS();
+
+	//Check Double Domination
+	CheckDoubleDomination();
+
+	CheckDomination();
+
+	//Sago: I just need to think why I placed this here... they should only spawn once
+	if(g_gametype.integer == GT_DOMINATION)
+		Team_Dom_SpawnPoints();
+
 	// see if it is time to end the level
 	CheckExitRules();
 
@@ -1839,4 +2484,12 @@ end = trap_Milliseconds();
 		}
 		trap_Cvar_Set("g_listEntity", "0");
 	}
+
+//unlagged - backward reconciliation #4
+	// record the time at the end of this frame - it should be about
+	// the time the next frame begins - when the server starts
+	// accepting commands from connected clients
+	level.frameStartTime = trap_Milliseconds();
+//unlagged - backward reconciliation #4
 }
+
