@@ -111,7 +111,10 @@ qboolean CheckGauntletAttack( gentity_t *ent ) {
 	}
 #endif
 
-	damage = 50 * s_quadFactor;
+	if(g_instantgib.integer)
+		damage = 1000; //High damage in instant gib (normally enough to gib)
+	else
+		damage = 50 * s_quadFactor;
 	G_Damage( traceEnt, ent, ent, forward, tr.endpos,
 		damage, 0, MOD_GAUNTLET );
 
@@ -137,6 +140,9 @@ rather than blindly truncating.  This prevents it from truncating
 into a wall.
 ======================
 */
+//unlagged - attack prediction #3
+// moved to q_shared.c
+/*
 void SnapVectorTowards( vec3_t v, vec3_t to ) {
 	int		i;
 
@@ -148,6 +154,33 @@ void SnapVectorTowards( vec3_t v, vec3_t to ) {
 		}
 	}
 }
+*/
+//unlagged - attack prediction #3
+
+//unlagged - attack prediction #3
+// moved from g_weapon.c
+/*
+======================
+SnapVectorTowards
+
+Round a vector to integers for more efficient network
+transmission, but make sure that it rounds towards a given point
+rather than blindly truncating.  This prevents it from truncating 
+into a wall.
+======================
+*/
+void SnapVectorTowards( vec3_t v, vec3_t to ) {
+	int		i;
+
+	for ( i = 0 ; i < 3 ; i++ ) {
+		if ( to[i] <= v[i] ) {
+			v[i] = (int)v[i];
+		} else {
+			v[i] = (int)v[i] + 1;
+		}
+	}
+}
+//unlagged - attack prediction #3
 
 #ifdef MISSIONPACK
 #define CHAINGUN_SPREAD		600
@@ -168,11 +201,25 @@ void Bullet_Fire (gentity_t *ent, float spread, int damage ) {
 	gentity_t	*traceEnt;
 	int			i, passent;
 
+//unlagged - attack prediction #2
+	// we have to use something now that the client knows in advance
+	int			seed = ent->client->attackTime % 256;
+//unlagged - attack prediction #2
+
 	damage *= s_quadFactor;
 
+//unlagged - attack prediction #2
+	// this has to match what's on the client
+/*
 	r = random() * M_PI * 2.0f;
 	u = sin(r) * crandom() * spread * 16;
 	r = cos(r) * crandom() * spread * 16;
+*/
+	r = Q_random(&seed) * M_PI * 2.0f;
+	u = sin(r) * Q_crandom(&seed) * spread * 16;
+	r = cos(r) * Q_crandom(&seed) * spread * 16;
+//unlagged - attack prediction #2
+
 	VectorMA (muzzle, 8192*16, forward, end);
 	VectorMA (end, r, right, end);
 	VectorMA (end, u, up, end);
@@ -180,6 +227,11 @@ void Bullet_Fire (gentity_t *ent, float spread, int damage ) {
 	passent = ent->s.number;
 	for (i = 0; i < 10; i++) {
 
+//unlagged - backward reconciliation #2
+		// backward-reconcile the other clients
+		G_DoTimeShiftFor( ent );
+//unlagged - backward reconciliation #2
+
 		trap_Trace (&tr, muzzle, NULL, NULL, end, passent, MASK_SHOT);
 		if ( tr.surfaceFlags & SURF_NOIMPACT ) {
 			return;
@@ -194,12 +246,22 @@ void Bullet_Fire (gentity_t *ent, float spread, int damage ) {
 		if ( traceEnt->takedamage && traceEnt->client ) {
 			tent = G_TempEntity( tr.endpos, EV_BULLET_HIT_FLESH );
 			tent->s.eventParm = traceEnt->s.number;
+//unlagged - attack prediction #2
+			// we need the client number to determine whether or not to
+			// suppress this event
+			tent->s.clientNum = ent->s.clientNum;
+//unlagged - attack prediction #2
 			if( LogAccuracyHit( traceEnt, ent ) ) {
 				ent->client->accuracy_hits++;
 			}
 		} else {
 			tent = G_TempEntity( tr.endpos, EV_BULLET_HIT_WALL );
 			tent->s.eventParm = DirToByte( tr.plane.normal );
+//unlagged - attack prediction #2
+			// we need the client number to determine whether or not to
+			// suppress this event
+			tent->s.clientNum = ent->s.clientNum;
+//unlagged - attack prediction #2
 		}
 		tent->s.otherEntityNum = ent->s.number;
 
@@ -327,6 +389,11 @@ void ShotgunPattern( vec3_t origin, vec3_t origin2, int seed, gentity_t *ent ) {
 	int			oldScore;
 	qboolean	hitClient = qfalse;
 
+//unlagged - attack prediction #2
+	// use this for testing
+	//Com_Printf( "Server seed: %d\n", seed );
+//unlagged - attack prediction #2
+
 	// derive the right and up vectors from the forward vector, because
 	// the client won't have any other information
 	VectorNormalize2( origin2, forward );
@@ -335,6 +402,11 @@ void ShotgunPattern( vec3_t origin, vec3_t origin2, int seed, gentity_t *ent ) {
 
 	oldScore = ent->client->ps.persistant[PERS_SCORE];
 
+//unlagged - backward reconciliation #2
+	// backward-reconcile the other clients
+	G_DoTimeShiftFor( ent );
+//unlagged - backward reconciliation #2
+
 	// generate the "random" spread pattern
 	for ( i = 0 ; i < DEFAULT_SHOTGUN_COUNT ; i++ ) {
 		r = Q_crandom( &seed ) * DEFAULT_SHOTGUN_SPREAD * 16;
@@ -347,6 +419,11 @@ void ShotgunPattern( vec3_t origin, vec3_t origin2, int seed, gentity_t *ent ) {
 			ent->client->accuracy_hits++;
 		}
 	}
+
+//unlagged - backward reconciliation #2
+	// put them back
+	G_UndoTimeShiftFor( ent );
+//unlagged - backward reconciliation #2
 }
 
 
@@ -357,7 +434,12 @@ void weapon_supershotgun_fire (gentity_t *ent) {
 	tent = G_TempEntity( muzzle, EV_SHOTGUN );
 	VectorScale( forward, 4096, tent->s.origin2 );
 	SnapVector( tent->s.origin2 );
-	tent->s.eventParm = rand() & 255;		// seed for spread pattern
+//Sago: This sound like a bad idea...
+//unlagged - attack prediction #2
+	// this has to be something the client can predict now
+	//tent->s.eventParm = rand() & 255;		// seed for spread pattern
+	tent->s.eventParm = ent->client->attackTime % 256; // seed for spread pattern
+//unlagged - attack prediction #2
 	tent->s.otherEntityNum = ent->s.number;
 
 	ShotgunPattern( tent->s.pos.trBase, tent->s.origin2, tent->s.eventParm, ent );
@@ -454,9 +536,16 @@ void weapon_railgun_fire (gentity_t *ent) {
 	gentity_t	*unlinkedEntities[MAX_RAIL_HITS];
 
 	damage = 100 * s_quadFactor;
+	if(g_instantgib.integer)
+		damage = 5000;
 
 	VectorMA (muzzle, 8192, forward, end);
 
+//unlagged - backward reconciliation #2
+	// backward-reconcile the other clients
+	G_DoTimeShiftFor( ent );
+//unlagged - backward reconciliation #2
+
 	// trace only against the solids, so the railgun will go through people
 	unlinked = 0;
 	hits = 0;
@@ -511,6 +600,11 @@ void weapon_railgun_fire (gentity_t *ent) {
 		unlinked++;
 	} while ( unlinked < MAX_RAIL_HITS );
 
+//unlagged - backward reconciliation #2
+	// put them back
+	G_UndoTimeShiftFor( ent );
+//unlagged - backward reconciliation #2
+
 	// link back in any entities we unlinked
 	for ( i = 0 ; i < unlinked ; i++ ) {
 		trap_LinkEntity( unlinkedEntities[i] );
@@ -618,14 +712,30 @@ void Weapon_LightningFire( gentity_t *ent ) {
 	gentity_t	*traceEnt, *tent;
 	int			damage, i, passent;
 
-	damage = 8 * s_quadFactor;
+//Sago: This sound like a workaround for the lightning killing people before they know it
+//unlagged - server options
+	// this is configurable now
+//	damage = 8 * s_quadFactor;
+	damage = g_lightningDamage.integer * s_quadFactor;
+//unlagged - server options
 
 	passent = ent->s.number;
 	for (i = 0; i < 10; i++) {
 		VectorMA( muzzle, LIGHTNING_RANGE, forward, end );
 
+//Sago: I'm not sure this should recieve backward reconciliation. It is not a real instant hit weapon, it can normally be dogded
+//unlagged - backward reconciliation #2
+	// backward-reconcile the other clients
+	G_DoTimeShiftFor( ent );
+//unlagged - backward reconciliation #2
+
 		trap_Trace( &tr, muzzle, NULL, NULL, end, passent, MASK_SHOT );
 
+//unlagged - backward reconciliation #2
+	// put them back
+	G_UndoTimeShiftFor( ent );
+//unlagged - backward reconciliation #2
+
 #ifdef MISSIONPACK
 		// if not the first trace (the lightning bounced of an invulnerability sphere)
 		if (i) {
@@ -810,6 +920,18 @@ FireWeapon
 ===============
 */
 void FireWeapon( gentity_t *ent ) {
+
+
+	//If elimination warmup, don't fire at all!
+	//if (g_gametype.integer == GT_ELIMINATION && level.roundStartTime>level.time )
+	//	return;
+
+	//Make people drop out of follow mode (this should be moved, so people can change betwean players.)
+	if (ent->client->sess.spectatorState == SPECTATOR_FOLLOW) {
+		StopFollowing( ent );
+		return;
+	}
+
 	if (ent->client->ps.powerups[PW_QUAD] ) {
 		s_quadFactor = g_quadfactor.value;
 	} else {
