@@ -32,16 +32,16 @@ MULTIPLAYER MENU (SERVER BROWSER)
 #include "ui_local.h"
 
 
-#define MAX_GLOBALSERVERS		128
+#define MAX_GLOBALSERVERS		124
 #define MAX_PINGREQUESTS		32
 #define MAX_ADDRESSLENGTH		64
 #define MAX_HOSTNAMELENGTH		22
 #define MAX_MAPNAMELENGTH		16
-#define MAX_LISTBOXITEMS		128
-#define MAX_LOCALSERVERS		128
+#define MAX_LISTBOXITEMS		124
+#define MAX_LOCALSERVERS		124
 #define MAX_STATUSLENGTH		64
 #define MAX_LEAGUELENGTH		28
-#define MAX_LISTBOXWIDTH		68
+#define MAX_LISTBOXWIDTH		62
 
 #define ART_BACK0				"menu/art/back_0"
 #define ART_BACK1				"menu/art/back_1"
@@ -59,7 +59,7 @@ MULTIPLAYER MENU (SERVER BROWSER)
 #define ART_UNKNOWNMAP			"menu/art/unknownmap"
 #define ART_REMOVE0				"menu/art/delete_0"
 #define ART_REMOVE1				"menu/art/delete_1"
-#define ART_PUNKBUSTER		"menu/art/pblogo"
+//#define ART_PUNKBUSTER		"menu/art/pblogo"
 
 #define ID_MASTER			10
 #define ID_GAMETYPE			11
@@ -75,7 +75,7 @@ MULTIPLAYER MENU (SERVER BROWSER)
 #define ID_CREATE			21
 #define ID_CONNECT			22
 #define ID_REMOVE			23
-#define ID_PUNKBUSTER 24
+//#define ID_PUNKBUSTER 24
 
 #define GR_LOGO				30
 #define GR_LETTERS			31
@@ -96,6 +96,11 @@ MULTIPLAYER MENU (SERVER BROWSER)
 #define GAMES_TEAMPLAY		2
 #define GAMES_TOURNEY		3
 #define GAMES_CTF			4
+#define GAMES_ELIMINATION		5
+#define GAMES_CTF_ELIMINATION		6
+#define GAMES_LMS			7
+#define GAMES_DOUBLE_D			8
+
 
 static const char *master_items[] = {
 	"Local",
@@ -110,6 +115,10 @@ static const char *servertype_items[] = {
 	"Team Deathmatch",
 	"Tournament",
 	"Capture the Flag",
+	"Elimination",
+	"CTF Elimination",
+	"Last Man Standing",
+	"Double Domination",
 	NULL
 };
 
@@ -131,7 +140,11 @@ static char* gamenames[] = {
 	"One Flag CTF",		// one flag ctf
 	"OverLoad",				// Overload
 	"Harvester",			// Harvester
-	"Rocket Arena 3",	// Rocket Arena 3
+	"Elimination",
+	"CTF Elimination",
+	"Last Man Standing",
+	"Double Domination",
+        "Rocket Arena 3",	// Rocket Arena 3
 	"Q3F",						// Q3F
 	"Urban Terror",		// Urban Terror
 	"OSP",						// Orange Smoothie Productions
@@ -142,13 +155,12 @@ static char* gamenames[] = {
 static char* netnames[] = {
 	"???",
 	"UDP",
-	"IPX",
 	NULL
 };
 
-static char quake3worldMessage[] = "Visit www.quake3world.com - News, Community, Events, Files";
+static char quake3worldMessage[] = "Visit www.openarena.ws - News, Community, Events, Files";
 
-const char* punkbuster_items[] = {
+/*const char* punkbuster_items[] = {
 	"Disabled",
 	"Enabled",
 	NULL
@@ -160,7 +172,7 @@ const char* punkbuster_msg[] = {
 	"Quake III Arena",
 	"is started.",
 	NULL
-};
+};*/
 
 typedef struct {
 	char	adrstr[MAX_ADDRESSLENGTH];
@@ -228,8 +240,8 @@ typedef struct {
 	char				favoriteaddresses[MAX_FAVORITESERVERS][MAX_ADDRESSLENGTH];
 	int					numfavoriteaddresses;
 
-	menulist_s		punkbuster;
-	menubitmap_s	pblogo;
+	//menulist_s		punkbuster;
+	//menubitmap_s	pblogo;
 } arenaservers_t;
 
 static arenaservers_t	g_arenaservers;
@@ -400,7 +412,7 @@ static void ArenaServers_UpdateMenu( void ) {
 			g_arenaservers.list.generic.flags		&= ~QMF_GRAYED;
 			g_arenaservers.refresh.generic.flags	&= ~QMF_GRAYED;
 			g_arenaservers.go.generic.flags			&= ~QMF_GRAYED;
-			g_arenaservers.punkbuster.generic.flags &= ~QMF_GRAYED;
+			//g_arenaservers.punkbuster.generic.flags &= ~QMF_GRAYED;
 
 			// update status bar
 			if( g_servertype == AS_GLOBAL || g_servertype == AS_MPLAYER ) {
@@ -427,7 +439,7 @@ static void ArenaServers_UpdateMenu( void ) {
 			g_arenaservers.list.generic.flags		|= QMF_GRAYED;
 			g_arenaservers.refresh.generic.flags	|= QMF_GRAYED;
 			g_arenaservers.go.generic.flags			|= QMF_GRAYED;
-			g_arenaservers.punkbuster.generic.flags |= QMF_GRAYED;
+			//g_arenaservers.punkbuster.generic.flags |= QMF_GRAYED;
 		}
 		else {
 			if( g_arenaservers.numqueriedservers < 0 ) {
@@ -454,7 +466,7 @@ static void ArenaServers_UpdateMenu( void ) {
 			g_arenaservers.list.generic.flags		|= QMF_GRAYED;
 			g_arenaservers.refresh.generic.flags	&= ~QMF_GRAYED;
 			g_arenaservers.go.generic.flags			|= QMF_GRAYED;
-			g_arenaservers.punkbuster.generic.flags &= ~QMF_GRAYED;
+			//g_arenaservers.punkbuster.generic.flags &= ~QMF_GRAYED;
 		}
 
 		// zero out list box
@@ -511,6 +523,30 @@ static void ArenaServers_UpdateMenu( void ) {
 				continue;
 			}
 			break;
+
+		case GAMES_ELIMINATION:
+			if( servernodeptr->gametype != GT_ELIMINATION ) {
+				continue;
+			}
+			break;
+		
+		case GAMES_CTF_ELIMINATION:
+			if( servernodeptr->gametype != GT_CTF_ELIMINATION ) {
+				continue;
+			}
+			break;
+
+		case GAMES_LMS:
+			if( servernodeptr->gametype != GT_LMS ) {
+				continue;
+			}
+			break;
+
+		case GAMES_DOUBLE_D:
+			if( servernodeptr->gametype != GT_DOUBLE_D ) {
+				continue;
+			}
+			break;
 		}
 
 		if( servernodeptr->pingtime < servernodeptr->minPing ) {
@@ -566,36 +602,38 @@ static void ArenaServers_Remove( void )
 	tableptr      = &g_arenaservers.table[g_arenaservers.list.curvalue];
 	servernodeptr = tableptr->servernode;
 
-	// find address in master list
+        // find address in master list
 	for (i=0; i<g_arenaservers.numfavoriteaddresses; i++)
-		if (!Q_stricmp(g_arenaservers.favoriteaddresses[i],servernodeptr->adrstr))
-				break;
-
-	// delete address from master list
-	if (i <= g_arenaservers.numfavoriteaddresses-1)
 	{
-		if (i < g_arenaservers.numfavoriteaddresses-1)
+		if (!Q_stricmp(g_arenaservers.favoriteaddresses[i],servernodeptr->adrstr))
 		{
-			// shift items up
-			memcpy( &g_arenaservers.favoriteaddresses[i], &g_arenaservers.favoriteaddresses[i+1], (g_arenaservers.numfavoriteaddresses - i - 1)*sizeof(MAX_ADDRESSLENGTH));
-		}
-		g_arenaservers.numfavoriteaddresses--;
+			// delete address from master list
+ 			if (i < g_arenaservers.numfavoriteaddresses-1)
+ 			{
+ 				// shift items up
+ 				memcpy( &g_arenaservers.favoriteaddresses[i], &g_arenaservers.favoriteaddresses[i+1], (g_arenaservers.numfavoriteaddresses - i - 1)* MAX_ADDRESSLENGTH );
+			}
+ 			g_arenaservers.numfavoriteaddresses--;
+ 			memset( &g_arenaservers.favoriteaddresses[g_arenaservers.numfavoriteaddresses], 0, MAX_ADDRESSLENGTH );
+ 			break;
+                }
 	}	
 
 	// find address in server list
 	for (i=0; i<g_numfavoriteservers; i++)
-		if (&g_favoriteserverlist[i] == servernodeptr)
-				break;
-
-	// delete address from server list
-	if (i <= g_numfavoriteservers-1)
 	{
-		if (i < g_numfavoriteservers-1)
+		if (&g_favoriteserverlist[i] == servernodeptr)
 		{
-			// shift items up
-			memcpy( &g_favoriteserverlist[i], &g_favoriteserverlist[i+1], (g_numfavoriteservers - i - 1)*sizeof(servernode_t));
-		}
-		g_numfavoriteservers--;
+			// delete address from server list
+ 			if (i < g_numfavoriteservers-1)
+ 			{
+ 				// shift items up
+ 				memcpy( &g_favoriteserverlist[i], &g_favoriteserverlist[i+1], (g_numfavoriteservers - i - 1)*sizeof(servernode_t));
+ 			}
+ 			g_numfavoriteservers--;
+ 			memset( &g_favoriteserverlist[ g_numfavoriteservers ], 0, sizeof(servernode_t));
+ 			break;
+                }
 	}	
 
 	g_arenaservers.numqueriedservers = g_arenaservers.numfavoriteaddresses;
@@ -645,7 +683,7 @@ static void ArenaServers_Insert( char* adrstr, char* info, int pingtime )
 	servernodeptr->pingtime   = pingtime;
 	servernodeptr->minPing    = atoi( Info_ValueForKey( info, "minPing") );
 	servernodeptr->maxPing    = atoi( Info_ValueForKey( info, "maxPing") );
-	servernodeptr->bPB = atoi( Info_ValueForKey( info, "punkbuster") );
+	//servernodeptr->bPB = atoi( Info_ValueForKey( info, "punkbuster") );
 
 	/*
 	s = Info_ValueForKey( info, "nettype" );
@@ -1038,6 +1076,22 @@ static void ArenaServers_StartRefresh( void )
 		case GAMES_CTF:
 			strcpy( myargs, " ctf" );
 			break;
+
+		case GAMES_ELIMINATION:
+			strcpy( myargs, " elimination" );
+			break;
+
+		case GAMES_CTF_ELIMINATION:
+			strcpy( myargs, " ctfelimination" );
+			break;
+
+		case GAMES_LMS:
+			strcpy( myargs, " lms" );
+			break;
+		
+		case GAMES_DOUBLE_D:
+			strcpy( myargs, " dd" );
+			break;
 		}
 
 
@@ -1154,7 +1208,7 @@ void ArenaServers_SetType( int type )
 PunkBuster_Confirm
 =================
 */
-static void Punkbuster_ConfirmEnable( qboolean result ) {
+/*static void Punkbuster_ConfirmEnable( qboolean result ) {
 	if (result)
 	{		
 		trap_SetPbClStatus(1);
@@ -1169,7 +1223,7 @@ static void Punkbuster_ConfirmDisable( qboolean result ) {
 		UI_Message( punkbuster_msg );
 	}
 	g_arenaservers.punkbuster.curvalue = Com_Clamp( 0, 1, trap_Cvar_VariableValue( "cl_punkbuster" ) );
-}
+}*/
 
 /*
 =================
@@ -1262,7 +1316,7 @@ static void ArenaServers_Event( void* ptr, int event ) {
 		ArenaServers_UpdateMenu();
 		break;
 	
-	case ID_PUNKBUSTER:
+	/*case ID_PUNKBUSTER:
 		if (g_arenaservers.punkbuster.curvalue)			
 		{
 			UI_ConfirmMenu_Style( "Enable Punkbuster?",  UI_CENTER|UI_INVERSE|UI_SMALLFONT, 0, Punkbuster_ConfirmEnable );
@@ -1271,7 +1325,7 @@ static void ArenaServers_Event( void* ptr, int event ) {
 		{
 			UI_ConfirmMenu_Style( "Disable Punkbuster?", UI_CENTER|UI_INVERSE|UI_SMALLFONT, 0, Punkbuster_ConfirmDisable );
 		}
-		break;
+		break;*/
 	}
 }
 
@@ -1312,6 +1366,14 @@ static sfxHandle_t ArenaServers_MenuKey( int key ) {
 		ArenaServers_StopRefresh();
 		ArenaServers_SaveChanges();
 	}
+        
+        if( key == K_MWHEELUP ) {
+            ScrollList_Key( &g_arenaservers.list, K_UPARROW );
+        }
+        
+        if( key == K_MWHEELDOWN ) {
+            ScrollList_Key( &g_arenaservers.list, K_DOWNARROW );
+        }
 
 
 	return Menu_DefaultKey( &g_arenaservers.menu, key );
@@ -1529,7 +1591,7 @@ static void ArenaServers_MenuInit( void ) {
 	g_arenaservers.go.height				= 64;
 	g_arenaservers.go.focuspic				= ART_CONNECT1;
 
-	g_arenaservers.punkbuster.generic.type			= MTYPE_SPINCONTROL;
+	/*g_arenaservers.punkbuster.generic.type			= MTYPE_SPINCONTROL;
 	g_arenaservers.punkbuster.generic.name			= "Punkbuster:";
 	g_arenaservers.punkbuster.generic.flags			= QMF_PULSEIFFOCUS|QMF_SMALLFONT;
 	g_arenaservers.punkbuster.generic.callback		= ArenaServers_Event;
@@ -1545,7 +1607,7 @@ static void ArenaServers_MenuInit( void ) {
 	g_arenaservers.pblogo.generic.y				= 176;
 	g_arenaservers.pblogo.width					= 32;
 	g_arenaservers.pblogo.height				= 16;
-	g_arenaservers.pblogo.errorpic				= ART_UNKNOWNMAP;
+	g_arenaservers.pblogo.errorpic				= ART_UNKNOWNMAP;*/
 	
 	Menu_AddItem( &g_arenaservers.menu, (void*) &g_arenaservers.banner );
 
@@ -1570,8 +1632,8 @@ static void ArenaServers_MenuInit( void ) {
 	Menu_AddItem( &g_arenaservers.menu, (void*) &g_arenaservers.create );
 	Menu_AddItem( &g_arenaservers.menu, (void*) &g_arenaservers.go );
 
-	Menu_AddItem( &g_arenaservers.menu, (void*) &g_arenaservers.punkbuster );
-	Menu_AddItem( &g_arenaservers.menu, (void*) &g_arenaservers.pblogo );
+	//Menu_AddItem( &g_arenaservers.menu, (void*) &g_arenaservers.punkbuster );
+	//Menu_AddItem( &g_arenaservers.menu, (void*) &g_arenaservers.pblogo );
 	
 	ArenaServers_LoadFavorites();
 
@@ -1582,7 +1644,7 @@ static void ArenaServers_MenuInit( void ) {
 		value--;
 	g_arenaservers.master.curvalue = value;
 
-	g_gametype = Com_Clamp( 0, 4, ui_browserGameType.integer );
+	g_gametype = Com_Clamp( 0, 5, ui_browserGameType.integer );
 	g_arenaservers.gametype.curvalue = g_gametype;
 
 	g_sortkey = Com_Clamp( 0, 4, ui_browserSortKey.integer );
@@ -1594,7 +1656,7 @@ static void ArenaServers_MenuInit( void ) {
 	g_emptyservers = Com_Clamp( 0, 1, ui_browserShowEmpty.integer );
 	g_arenaservers.showempty.curvalue = g_emptyservers;
 	
-	g_arenaservers.punkbuster.curvalue = Com_Clamp( 0, 1, trap_Cvar_VariableValue( "cl_punkbuster" ) );
+	//g_arenaservers.punkbuster.curvalue = Com_Clamp( 0, 1, trap_Cvar_VariableValue( "cl_punkbuster" ) );
 
 	// force to initial state and refresh
 	type = g_servertype;
@@ -1625,7 +1687,7 @@ void ArenaServers_Cache( void ) {
 	trap_R_RegisterShaderNoMip( ART_ARROWS_UP );
 	trap_R_RegisterShaderNoMip( ART_ARROWS_DOWN );
 	trap_R_RegisterShaderNoMip( ART_UNKNOWNMAP );
-	trap_R_RegisterShaderNoMip( ART_PUNKBUSTER );
+	//trap_R_RegisterShaderNoMip( ART_PUNKBUSTER );
 }
 
 
